/* linux/drivers/media/video/5k6aa.h
 *
 * Copyright (c) 2010 Samsung Electronics Co., Ltd.
 * 		http://www.samsung.com/
 *
 * Driver for S5K6AA (SXGA camera) from Samsung Electronics
 * supporting MIPI CSI-2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#define S5K6AA_COMPLETE
#undef S5K6AA_COMPLETE
#ifndef __S5K6AA_H__
#define __S5K6AA_H__

struct s5k6aa_reg {
	unsigned short addr;
	unsigned short val;
};

struct s5k6aa_regset_type {
	unsigned char *regset;
	int len;
};

/*
 * Macro
 */
#define REGSET_LENGTH(x)	(sizeof(x)/sizeof(s5k6aa_reg))

/*
 * Host S/W Register interface (0x70000000-0x70002000)
 */
/* Initialization section */
#define REG_TC_IPRM_InClockLSBs			0x01B8
#define REG_TC_IPRM_InClockMSBs			0x01BA
#define REG_TC_IPRM_PllFreqDiv4			0x01BC
#define REG_TC_IPRM_AddHeader			0x01BE
#define REG_TC_IPRM_ValidVActiveLow		0x01C0
#define REG_TC_IPRM_SenI2CAdr			0x01C2
#define REG_TC_IPRM_MI2CDrSclMan		0x01C4
#define REG_TC_IPRM_UseNPviClocks		0x01C6
#define REG_TC_IPRM_UseNMipiClocks		0x01C8
#define REG_TC_IPRM_bBlockInternalPllCalc	0x01CA

#define REG_TC_IPRM_OpClk4KHz_0			0x01CC
#define REG_TC_IPRM_MinOutRate4KHz_0		0x01CE
#define REG_TC_IPRM_MaxOutRate4KHz_0		0x01D0

#define REG_TC_IPRM_OpClk4KHz_1			0x01D2
#define REG_TC_IPRM_MinOutRate4KHz_1		0x01D4
#define REG_TC_IPRM_MaxOutRate4KHz_1		0x01D6

#define REG_TC_IPRM_OpClk4KHz_2			0x01D8
#define REG_TC_IPRM_MinOutRate4KHz_2		0x01DA
#define REG_TC_IPRM_MaxOutRate4KHz_2		0x01DC

#define REG_TC_IPRM_UseRegsAPI			0x01DE
#define REG_TC_IPRM_InitParamsUpdated		0x01E0
#define REG_TC_IPRM_ErrorInfo			0x01E2

/* Preview control section */
#define REG_0TC_PCFG_usWidth			0x0242
#define REG_0TC_PCFG_usHeight			0x0244
#define REG_0TC_PCFG_Format			0x0246
#define REG_0TC_PCFG_usMaxOut4KHzRate		0x0248
#define REG_0TC_PCFG_usMinOut4KHzRate		0x024A
#define REG_0TC_PCFG_PVIMask			0x024C
#define REG_0TC_PCFG_uClockInd			0x024E
#define REG_0TC_PCFG_usFrTimeType		0x0250
#define REG_0TC_PCFG_FrRateQualityType		0x0252
#define REG_0TC_PCFG_usMaxFrTimeMsecMult10	0x0254
#define REG_0TC_PCFG_usMinFrTimeMsecMult10	0x0256
#define REG_0TC_PCFG_sSaturation		0x0258
#define REG_0TC_PCFG_sSharpBlur			0x025A
#define REG_0TC_PCFG_sGlamour			0x025C
#define REG_0TC_PCFG_sColorTemp			0x025E
#define REG_0TC_PCFG_uDeviceGammaIndex		0x0260
#define REG_0TC_PCFG_uPrevMirror		0x0262
#define REG_0TC_PCFG_uCaptureMirror		0x0264
#define REG_0TC_PCFG_uRotation			0x0266

#define REG_1TC_PCFG_usWidth			0x0268
#define REG_1TC_PCFG_usHeight			0x026A
#define REG_1TC_PCFG_Format			0x026C
#define REG_1TC_PCFG_usMaxOut4KHzRate		0x026E
#define REG_1TC_PCFG_usMinOut4KHzRate		0x0270
#define REG_1TC_PCFG_PVIMask			0x0272
#define REG_1TC_PCFG_uClockInd			0x0274
#define REG_1TC_PCFG_usFrTimeType		0x0276
#define REG_1TC_PCFG_FrRateQualityType		0x0278
#define REG_1TC_PCFG_usMaxFrTimeMsecMult10	0x027A
#define REG_1TC_PCFG_usMinFrTimeMsecMult10	0x027C
#define REG_1TC_PCFG_sSaturation		0x027E
#define REG_1TC_PCFG_sSharpBlur			0x0280
#define REG_1TC_PCFG_sGlamour			0x0282
#define REG_1TC_PCFG_sColorTemp			0x0284
#define REG_1TC_PCFG_uDeviceGammaIndex		0x0286
#define REG_1TC_PCFG_uPrevMirror		0x0288
#define REG_1TC_PCFG_uCaptureMirror		0x028A
#define REG_1TC_PCFG_uRotation			0x028C

#define REG_2TC_PCFG_usWidth			0x028E
#define REG_2TC_PCFG_usHeight			0x0290
#define REG_2TC_PCFG_Format			0x0292
#define REG_2TC_PCFG_usMaxOut4KHzRate		0x0294
#define REG_2TC_PCFG_usMinOut4KHzRate		0x0296
#define REG_2TC_PCFG_PVIMask			0x0298
#define REG_2TC_PCFG_uClockInd			0x029A
#define REG_2TC_PCFG_usFrTimeType		0x029C
#define REG_2TC_PCFG_FrRateQualityType		0x029E
#define REG_2TC_PCFG_usMaxFrTimeMsecMult10	0x02A0
#define REG_2TC_PCFG_usMinFrTimeMsecMult10	0x02A2
#define REG_2TC_PCFG_sSaturation		0x02A4
#define REG_2TC_PCFG_sSharpBlur			0x02A6
#define REG_2TC_PCFG_sGlamour			0x02A8
#define REG_2TC_PCFG_sColorTemp			0x02AA
#define REG_2TC_PCFG_uDeviceGammaIndex		0x02AC
#define REG_2TC_PCFG_uPrevMirror		0x02AE
#define REG_2TC_PCFG_uCaptureMirror		0x02B0
#define REG_2TC_PCFG_uRotation			0x02B2

#define REG_3TC_PCFG_usWidth			0x02B4
#define REG_3TC_PCFG_usHeight			0x02B6
#define REG_3TC_PCFG_Format			0x02B8
#define REG_3TC_PCFG_usMaxOut4KHzRate		0x02BA
#define REG_3TC_PCFG_usMinOut4KHzRate		0x02BC
#define REG_3TC_PCFG_PVIMask			0x02BE
#define REG_3TC_PCFG_uClockInd			0x02C0
#define REG_3TC_PCFG_usFrTimeType		0x02C2
#define REG_3TC_PCFG_FrRateQualityType		0x02C4
#define REG_3TC_PCFG_usMaxFrTimeMsecMult10	0x02C6
#define REG_3TC_PCFG_usMinFrTimeMsecMult10	0x02C8
#define REG_3TC_PCFG_sSaturation		0x02CA
#define REG_3TC_PCFG_sSharpBlur			0x02CC
#define REG_3TC_PCFG_sGlamour			0x02CE
#define REG_3TC_PCFG_sColorTemp			0x02D0
#define REG_3TC_PCFG_uDeviceGammaIndex		0x02D2
#define REG_3TC_PCFG_uPrevMirror		0x02D4
#define REG_3TC_PCFG_uCaptureMirror		0x02D6
#define REG_3TC_PCFG_uRotation			0x02D8

#define REG_4TC_PCFG_usWidth			0x02DA
#define REG_4TC_PCFG_usHeight			0x02DC
#define REG_4TC_PCFG_Format			0x02DE
#define REG_4TC_PCFG_usMaxOut4KHzRate		0x02E0
#define REG_4TC_PCFG_usMinOut4KHzRate		0x02E2
#define REG_4TC_PCFG_PVIMask			0x02E4
#define REG_4TC_PCFG_uClockInd			0x02E6
#define REG_4TC_PCFG_usFrTimeType		0x02E8
#define REG_4TC_PCFG_FrRateQualityType		0x02EA
#define REG_4TC_PCFG_usMaxFrTimeMsecMult10	0x02EC
#define REG_4TC_PCFG_usMinFrTimeMsecMult10	0x02EE
#define REG_4TC_PCFG_sSaturation		0x02F0
#define REG_4TC_PCFG_sSharpBlur			0x02F2
#define REG_4TC_PCFG_sGlamour			0x02F4
#define REG_4TC_PCFG_sColorTemp			0x02F6
#define REG_4TC_PCFG_uDeviceGammaIndex		0x02F8
#define REG_4TC_PCFG_uPrevMirror		0x02FA
#define REG_4TC_PCFG_uCaptureMirror		0x02FC
#define REG_4TC_PCFG_uRotation			0x02FEa

#define REG_AC_TC_PCFG_usWidth			0x11C8
#define REG_AC_TC_PCFG_usHeight			0x11CA
#define REG_AC_TC_PCFG_Format			0x11CC
#define REG_AC_TC_PCFG_usMaxOut4KHzRate		0x11CE
#define REG_AC_TC_PCFG_usMinOut4KHzRate		0x11D0
#define REG_AC_TC_PCFG_PVIMask			0x11D2
#define REG_AC_TC_PCFG_uClockInd		0x11D4
#define REG_AC_TC_PCFG_usFrTimeType		0x11D6
#define REG_AC_TC_PCFG_FrRateQualityType	0x11D8
#define REG_AC_TC_PCFG_usMaxFrTimeMsecMult10	0x11DA
#define REG_AC_TC_PCFG_usMinFrTimeMsecMult10	0x11DC
#define REG_AC_TC_PCFG_sSaturation		0x11DE
#define REG_AC_TC_PCFG_sSharpBlur		0x11E0
#define REG_AC_TC_PCFG_sGlamour			0x11E2
#define REG_AC_TC_PCFG_sColorTemp		0x11E4
#define REG_AC_TC_PCFG_uDeviceGammaIndex	0x11E6
#define REG_AC_TC_PCFG_uPrevMirror		0x11E8
#define REG_AC_TC_PCFG_uCaptureMirror		0x11EA
#define REG_AC_TC_PCFG_uRotation		0x11EC

/* Capture control section */
#define REG_0TC_CCFG_uCaptureMode		0x030C
#define REG_0TC_CCFG_usWidth			0x030E
#define REG_0TC_CCFG_usHeight			0x0310
#define REG_0TC_CCFG_Format			0x3012
#define REG_0TC_CCFG_usMaxOut4KHzRate		0x0314
#define REG_0TC_CCFG_usMinOut4KHzRate		0x0316
#define REG_0TC_CCFG_PVIMask			0x0318
#define REG_0TC_CCFG_uClockInd			0x031A
#define REG_0TC_CCFG_usFrTimeType		0x031C
#define REG_0TC_CCFG_FrRateQualityType		0x031E
#define REG_0TC_CCFG_usMaxFrTimeMsecMult10	0x0320
#define REG_0TC_CCFG_usMinFrTimeMsecMult10	0x0322
#define REG_0TC_CCFG_sSaturation		0x0324
#define REG_0TC_CCFG_sSharpBlur			0x0326
#define REG_0TC_CCFG_sGlamour			0x0328
#define REG_0TC_CCFG_sColorTemp			0x032A
#define REG_0TC_CCFG_uDeviceGammaIndex		0x032C

#define REG_1TC_CCFG_uCaptureMode		0x032E
#define REG_1TC_CCFG_usWidth			0x0330
#define REG_1TC_CCFG_usHeight			0x0332
#define REG_1TC_CCFG_Format			0x0334
#define REG_1TC_CCFG_usMaxOut4KHzRate		0x0336
#define REG_1TC_CCFG_usMinOut4KHzRate		0x0338
#define REG_1TC_CCFG_PVIMask			0x033A
#define REG_1TC_CCFG_uClockInd			0x033C
#define REG_1TC_CCFG_usFrTimeType		0x033E
#define REG_1TC_CCFG_FrRateQualityType		0x0340
#define REG_1TC_CCFG_usMaxFrTimeMsecMult10	0x0342
#define REG_1TC_CCFG_usMinFrTimeMsecMult10	0x0344
#define REG_1TC_CCFG_sSaturation		0x0346
#define REG_1TC_CCFG_sSharpBlur			0x0348
#define REG_1TC_CCFG_sGlamour			0x034A
#define REG_1TC_CCFG_sColorTemp			0x034C
#define REG_1TC_CCFG_uDeviceGammaIndex		0x034E

#define REG_2TC_CCFG_uCaptureMode		0x0350
#define REG_2TC_CCFG_usWidth			0x0352
#define REG_2TC_CCFG_usHeight			0x0354
#define REG_2TC_CCFG_Format			0x0356
#define REG_2TC_CCFG_usMaxOut4KHzRate		0x0358
#define REG_2TC_CCFG_usMinOut4KHzRate		0x035A
#define REG_2TC_CCFG_PVIMask			0x035C
#define REG_2TC_CCFG_uClockInd			0x035E
#define REG_2TC_CCFG_usFrTimeType		0x0360
#define REG_2TC_CCFG_FrRateQualityType		0x0362
#define REG_2TC_CCFG_usMaxFrTimeMsecMult10	0x0364
#define REG_2TC_CCFG_usMinFrTimeMsecMult10	0x0366
#define REG_2TC_CCFG_sSaturation		0x0368
#define REG_2TC_CCFG_sSharpBlur			0x036A
#define REG_2TC_CCFG_sGlamour			0x036C
#define REG_2TC_CCFG_sColorTemp			0x036E
#define REG_2TC_CCFG_uDeviceGammaIndex		0x0370

#define REG_3TC_CCFG_uCaptureMode		0x0372
#define REG_3TC_CCFG_usWidth			0x0374
#define REG_3TC_CCFG_usHeight			0x0376
#define REG_3TC_CCFG_Format			0x0378
#define REG_3TC_CCFG_usMaxOut4KHzRate		0x037A
#define REG_3TC_CCFG_usMinOut4KHzRate		0x037C
#define REG_3TC_CCFG_PVIMask			0x037E
#define REG_3TC_CCFG_uClockInd			0x0380
#define REG_3TC_CCFG_usFrTimeType		0x0382
#define REG_3TC_CCFG_FrRateQualityType		0x0384
#define REG_3TC_CCFG_usMaxFrTimeMsecMult10	0x0386
#define REG_3TC_CCFG_usMinFrTimeMsecMult10	0x0388
#define REG_3TC_CCFG_sSaturation		0x038A
#define REG_3TC_CCFG_sSharpBlur			0x038C
#define REG_3TC_CCFG_sGlamour			0x038E
#define REG_3TC_CCFG_sColorTemp			0x0390
#define REG_3TC_CCFG_uDeviceGammaIndex		0x0392

#define REG_4TC_CCFG_uCaptureMode		0x0394
#define REG_4TC_CCFG_usWidth			0x0396
#define REG_4TC_CCFG_usHeight			0x0398
#define REG_4TC_CCFG_Format			0x039A
#define REG_4TC_CCFG_usMaxOut4KHzRate		0x039C
#define REG_4TC_CCFG_usMinOut4KHzRate		0x039E
#define REG_4TC_CCFG_PVIMask			0x03A0
#define REG_4TC_CCFG_uClockInd			0x03A2
#define REG_4TC_CCFG_usFrTimeType		0x03A4
#define REG_4TC_CCFG_FrRateQualityType		0x03A6
#define REG_4TC_CCFG_usMaxFrTimeMsecMult10	0x03A8
#define REG_4TC_CCFG_usMinFrTimeMsecMult10	0x03AA
#define REG_4TC_CCFG_sSaturation		0x03AC
#define REG_4TC_CCFG_sSharpBlur			0x03AE
#define REG_4TC_CCFG_sGlamour			0x03B0
#define REG_4TC_CCFG_sColorTemp			0x03B2
#define REG_4TC_CCFG_uDeviceGammaIndex		0x03B4

#define REG_AC_TC_CCFG_uCaptureMode		0x1214
#define REG_AC_TC_CCFG_usWidth			0x1216
#define REG_AC_TC_CCFG_usHeight			0x1218
#define REG_AC_TC_CCFG_Format			0x121A
#define REG_AC_TC_CCFG_usMaxOut4KHzRate		0x121C
#define REG_AC_TC_CCFG_usMinOut4KHzRate		0x121E
#define REG_AC_TC_CCFG_PVIMask			0x1220
#define REG_AC_TC_CCFG_uClockInd		0x1222
#define REG_AC_TC_CCFG_usFrTimeType		0x1224
#define REG_AC_TC_CCFG_FrRateQualityType	0x1226
#define REG_AC_TC_CCFG_usMaxFrTimeMsecMult10	0x1228
#define REG_AC_TC_CCFG_usMinFrTimeMsecMult10	0x122A
#define REG_AC_TC_CCFG_sSaturation		0x122C
#define REG_AC_TC_CCFG_sSharpBlur		0x122E
#define REG_AC_TC_CCFG_sGlamour			0x1230
#define REG_AC_TC_CCFG_sColorTemp		0x1232
#define REG_AC_TC_CCFG_uDeviceGammaIndex	0x1234

/* Configuration value section */
/* Frame rate */
/* for REG_[NUM]TC_PCFG_usFrTimeType */
#define TC_FR_TIME_DYNAMIC			0
#define TC_FR_TIME_FIXED_NOT_ACCURATE		1
#define TC_FR_TIME_FIXED_ACCURATE		2
/* for REG_[NUM]TC_PCFG_FrRateQualityType */
#define TC_FRVSQ_BEST_FRRATE			1	/* Binning enabled */
#define TC_FRVSQ_BEST_QUALITY			2 	/* Binning disabled */

/* General purpose section */
#define REG_TC_GP_SpecialEffects		0x01EE
#define REG_TC_GP_EnablePreview			0x01F0
#define REG_TC_GP_EnablePreviewChanged		0x01F2
#define REG_TC_GP_EnableCapture			0x01F4
#define REG_TC_GP_EnableCaptureChanged		0x01F6
#define REG_TC_GP_NewConfigSync			0x01F8
#define REG_TC_GP_PrevReqInputWidth		0x01FA
#define REG_TC_GP_PrevReqInputHeight		0x01FC
#define REG_TC_GP_PrevInputWidthOfs		0x01FE
#define REG_TC_GP_PrevInputHeightOfs		0x0200
#define REG_TC_GP_CapReqInputWidth		0x0202
#define REG_TC_GP_CapReqInputHeight		0x0204
#define REG_TC_GP_CapInputWidthOfs		0x0206
#define REG_TC_GP_CapInputHeightOfs		0x0208
#define REG_TC_GP_PrevZoomReqInputWidth		0x020A
#define REG_TC_GP_PrevZoomReqInputHeight	0x020C
#define REG_TC_GP_PrevZoomReqInputWidthOfs	0x020E
#define REG_TC_GP_PrevZoomReqInputHeightOfs	0x0210
#define REG_TC_GP_CapZoomReqInputWidth		0x0212
#define REG_TC_GP_CapZoomReqInputHeight		0x0214
#define REG_TC_GP_CapZoomReqInputWidthOfs	0x0216
#define REG_TC_GP_CapZoomReqInputHeightOfs	0x0218
#define REG_TC_GP_InputsChangeRequest		0x021A
#define REG_TC_GP_ActivePrevConfig		0x021C
#define REG_TC_GP_PrevConfigChanged		0x021E
#define REG_TC_GP_PrevOpenAfterChange		0x0220
#define REG_TC_GP_ErrorPrevConfig		0x0222
#define REG_TC_GP_ActiveCapConfig		0x0224
#define REG_TC_GP_CapConfigChanged		0x0226
#define REG_TC_GP_ErrorCapConfig		0x0228
#define REG_TC_GP_PrevConfigBypassChanged	0x022A
#define REG_TC_GP_CapConfigBypassChanged	0x022C
#define REG_TC_GP_SleepMode			0x022E
#define REG_TC_GP_SleepModeChanged		0x0230
#define REG_TC_GP_SRA_AddLow			0x0232
#define REG_TC_GP_SRA_AddHigh			0x0234
#define REG_TC_GP_SRA_AccessType		0x0236
#define REG_TC_GP_SRA_Changed			0x0238
#define REG_TC_GP_PrevMinFrTimeMsecMult10	0x023A
#define REG_TC_GP_PrevOutKHzRate		0x023C
#define REG_TC_GP_CapMinFrTimeMsecMult10	0x023E
#define REG_TC_GP_CapOutKHzRate			0x0240

/* Image property control section */
#define REG_TC_UserBrightness			0x01E4
#define REG_TC_UserContrast			0x01E6
#define REG_TC_UserSaturation			0x01E8
#define REG_TC_UserSharpBlur			0x01EA
#define REG_TC_UserGlamour			0x01EC

/* Flash control section */
#define REG_TC_FLS_Mode				0x03B6
#define REG_TC_FLS_Threshold			0x03B8
#define REG_TC_FLS_Polarity			0x03BA
#define REG_TC_FLS_XenonMode			0x03BC
#define REG_TC_FLS_XenonPreFlashCnt		0x03BE

/* Extended image property control section */
#define REG_SF_USER_LeiLow			0x03C0
#define REG_SF_USER_LeiHigh			0x03C2
#define REG_SF_USER_LeiChanged			0x03C4
#define REG_SF_USER_Exposure			0x03C6
#define REG_SF_USER_ExposureChanged		0x03CA
#define REG_SF_USER_TotalGain			0x03CC
#define REG_SF_USER_TotalGainChanged		0x03CE
#define REG_SF_USER_Rgain			0x03D0
#define REG_SF_USER_RgainChanged		0x03D2
#define REG_SF_USER_Ggain			0x03D4
#define REG_SF_USER_GgainChanged		0x03D6
#define REG_SF_USER_Bgain			0x03D8
#define REG_SF_USER_BgainChanged		0x03DA
#define REG_SF_USER_FlickerQuant		0x03DC
#define REG_SF_USER_FlickerQuantChanged		0x03DE
#define REG_SF_USER_GASRAlphaVal		0x03E0
#define REG_SF_USER_GASRAlphaChanged		0x03E2
#define REG_SF_USER_GASGAlphaVal		0x03E4
#define REG_SF_USER_GASGAlphaChanged		0x03E6
#define REG_SF_USER_GASBAlphaVal		0x03E8
#define REG_SF_USER_GASBAlphaChanged		0x03EA
#define REG_SF_USER_DbgIdx			0x03EC
#define REG_SF_USER_DbgVal			0x03EE
#define REG_SF_USER_DbgChanged			0x03F0
#define REG_SF_USER_aGain			0x03F2
#define REG_SF_USER_aGainChanged		0x03F4
#define REG_SF_USER_dGain			0x03F6
#define REG_SF_USER_dGainChanged		0x03F8

/* Output interface control section */
#define REG_TC_OIF_EnMipiLanes			0x03FA
#define REG_TC_OIF_EnPackets			0x03FC
#define REG_TC_OIF_CfgChanged			0x03FE

/* Debug control section */
#define REG_TC_DBG_AutoAlgEnBits		0x0400
#define REG_TC_DBG_IspBypass			0x0402
#define REG_TC_DBG_ReInitCmd			0x0404

/* Version information section */
#define REG_FWdate				0x012C
#define REG_FWapiVer				0x012E
#define REG_FWrevision				0x0130
#define REG_FWpid				0x0132
#define REG_FWprjName				0x0134
#define REG_FWcompDate				0x0140
#define REG_FWSFC_VER				0x014C
#define REG_FWTC_VER				0x014E
#define REG_FWrealImageLine			0x0150
#define REG_FWsenId				0x0152
#define REG_FWusDevIdQaVersion			0x0154
#define REG_FWusFwCompilationBits		0x0156
#define REG_ulSVNrevision			0x0158
#define REG_SVNpathRomAddress			0x015C
#define REG_TRAP_N_PATCH_START_ADD		0x1B00

#define	setot_usForceClocksSettings		0x0AEA
#define	setot_usConfigClocksSettings		0x0AEC

/*
 * User defined commands
 */
/* S/W defined features for tune */
#define REG_DELAY	0xFF	/* in ms */
#define REG_CMD		0xFFFF	/* Followed by command */

/* Following order should not be changed */
enum image_size_s5k6aa {
	/* This SoC supports upto SXGA (1280*1024) */
#if 0
	QQVGA,	/* 160*120*/
	QCIF,	/* 176*144 */
	QVGA,	/* 320*240 */
	CIF,	/* 352*288 */
#endif
	VGA,	/* 640*480 */
#if 0
	SVGA,	/* 800*600 */
	HD720P,	/* 1280*720 */
	SXGA,	/* 1280*1024 */
#endif
};

/*
 * Following values describe controls of camera
 * in user aspect and must be match with index of s5k6aa_regset[]
 * These values indicates each controls and should be used
 * to control each control
 */
enum s5k6aa_control {
	S5K6AA_INIT,
	S5K6AA_EV,
	S5K6AA_AWB,
	S5K6AA_MWB,
	S5K6AA_EFFECT,
	S5K6AA_CONTRAST,
	S5K6AA_SATURATION,
	S5K6AA_SHARPNESS,
};

#define S5K6AA_REGSET(x)	{	\
	.regset = x,			\
	.len = sizeof(x)/sizeof(s5k6aa_reg),}

/*
 * User tuned register setting values
 */
unsigned char s5k6aa_init_reg[][4] = {
	/*******************/
	{0xFC, 0xFC, 0xD0, 0x00},
	/*******************/
	{0x00, 0x10, 0x00, 0x01},/* Reset */
	{0x10, 0x30, 0x00, 0x00},/* Clear host interrupt so main will wait */
	{0x00, 0x14, 0x00, 0x01},/* ARM go */

	/*******************/
	/* Delay{1000},p100*/
	/* Wait100mSec */
	/* Start T&P part */

	{REG_DELAY, 100, 0, 0},

	{0x00, 0x28, 0x70, 0x00},
	{0x00, 0x2A, 0x1D, 0x60},
	{0x0F, 0x12, 0xB5, 0x70},
	{0x0F, 0x12, 0x49, 0x28},
	{0x0F, 0x12, 0x48, 0x28},
	{0x0F, 0x12, 0x22, 0x05},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF9, 0x22},
	{0x0F, 0x12, 0x49, 0x27},
	{0x0F, 0x12, 0x20, 0x02},
	{0x0F, 0x12, 0x83, 0xC8},
	{0x0F, 0x12, 0x20, 0x01},
	{0x0F, 0x12, 0x31, 0x20},
	{0x0F, 0x12, 0x80, 0x88},
	{0x0F, 0x12, 0x49, 0x25},
	{0x0F, 0x12, 0x48, 0x26},
	{0x0F, 0x12, 0x22, 0x04},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF9, 0x17},
	{0x0F, 0x12, 0x49, 0x25},
	{0x0F, 0x12, 0x48, 0x25},
	{0x0F, 0x12, 0x22, 0x06},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF9, 0x12},
	{0x0F, 0x12, 0x49, 0x24},
	{0x0F, 0x12, 0x48, 0x25},
	{0x0F, 0x12, 0x22, 0x07},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF9, 0x0D},
	{0x0F, 0x12, 0x49, 0x24},
	{0x0F, 0x12, 0x48, 0x24},
	{0x0F, 0x12, 0x22, 0x08},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF9, 0x08},
	{0x0F, 0x12, 0x49, 0x23},
	{0x0F, 0x12, 0x48, 0x24},
	{0x0F, 0x12, 0x22, 0x09},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF9, 0x03},
	{0x0F, 0x12, 0x49, 0x23},
	{0x0F, 0x12, 0x48, 0x23},
	{0x0F, 0x12, 0x60, 0xC1},
	{0x0F, 0x12, 0x68, 0x82},
	{0x0F, 0x12, 0x1A, 0x51},
	{0x0F, 0x12, 0x82, 0x01},
	{0x0F, 0x12, 0x4C, 0x22},
	{0x0F, 0x12, 0x26, 0x07},
	{0x0F, 0x12, 0x68, 0x21},
	{0x0F, 0x12, 0x07, 0x36},
	{0x0F, 0x12, 0x42, 0xB1},
	{0x0F, 0x12, 0xDA, 0x05},
	{0x0F, 0x12, 0x48, 0x20},
	{0x0F, 0x12, 0x22, 0xD8},
	{0x0F, 0x12, 0x1C, 0x05},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xFA},
	{0x0F, 0x12, 0x60, 0x25},
	{0x0F, 0x12, 0x68, 0xA1},
	{0x0F, 0x12, 0x42, 0xB1},
	{0x0F, 0x12, 0xDA, 0x07},
	{0x0F, 0x12, 0x48, 0x1B},
	{0x0F, 0x12, 0x22, 0x24},
	{0x0F, 0x12, 0x38, 0x24},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xF1},
	{0x0F, 0x12, 0x48, 0x19},
	{0x0F, 0x12, 0x38, 0x24},
	{0x0F, 0x12, 0x60, 0xA0},
	{0x0F, 0x12, 0x4D, 0x18},
	{0x0F, 0x12, 0x6D, 0x29},
	{0x0F, 0x12, 0x42, 0xB1},
	{0x0F, 0x12, 0xDA, 0x07},
	{0x0F, 0x12, 0x48, 0x15},
	{0x0F, 0x12, 0x22, 0x8F},
	{0x0F, 0x12, 0x00, 0xD2},
	{0x0F, 0x12, 0x30, 0xD8},
	{0x0F, 0x12, 0x1C, 0x04},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xE3},
	{0x0F, 0x12, 0x65, 0x2C},
	{0x0F, 0x12, 0xBC, 0x70},
	{0x0F, 0x12, 0xBC, 0x08},
	{0x0F, 0x12, 0x47, 0x18},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x1F, 0x53},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x12, 0x7B},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x03, 0x98},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x1E, 0x4D},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x89, 0x0D},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x1E, 0x73},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x27, 0xA9},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x1E, 0x91},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x27, 0xC5},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x1E, 0xF7},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x28, 0x5F},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x1E, 0xB3},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x28, 0xFF},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x20, 0x6C},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x04, 0xAC},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x06, 0xCC},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x23, 0xA4},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x07, 0x04},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0xB5, 0x10},
	{0x0F, 0x12, 0x1C, 0x04},
	{0x0F, 0x12, 0x48, 0x4D},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xBB},
	{0x0F, 0x12, 0x4A, 0x4D},
	{0x0F, 0x12, 0x4B, 0x4D},
	{0x0F, 0x12, 0x88, 0x11},
	{0x0F, 0x12, 0x88, 0x5B},
	{0x0F, 0x12, 0x88, 0x52},
	{0x0F, 0x12, 0x43, 0x59},
	{0x0F, 0x12, 0x18, 0x89},
	{0x0F, 0x12, 0x42, 0x88},
	{0x0F, 0x12, 0xD8, 0x00},
	{0x0F, 0x12, 0x1C, 0x08},
	{0x0F, 0x12, 0x60, 0x20},
	{0x0F, 0x12, 0xBC, 0x10},
	{0x0F, 0x12, 0xBC, 0x08},
	{0x0F, 0x12, 0x47, 0x18},
	{0x0F, 0x12, 0xB5, 0x10},
	{0x0F, 0x12, 0x1C, 0x04},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xB1},
	{0x0F, 0x12, 0x49, 0x44},
	{0x0F, 0x12, 0x89, 0x89},
	{0x0F, 0x12, 0x43, 0x48},
	{0x0F, 0x12, 0x02, 0x00},
	{0x0F, 0x12, 0x0C, 0x00},
	{0x0F, 0x12, 0x21, 0x01},
	{0x0F, 0x12, 0x03, 0x49},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xB0},
	{0x0F, 0x12, 0x60, 0x20},
	{0x0F, 0x12, 0xE7, 0xED},
	{0x0F, 0x12, 0xB5, 0x10},
	{0x0F, 0x12, 0x1C, 0x04},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xB2},
	{0x0F, 0x12, 0x68, 0x21},
	{0x0F, 0x12, 0x04, 0x09},
	{0x0F, 0x12, 0x0C, 0x09},
	{0x0F, 0x12, 0x1A, 0x40},
	{0x0F, 0x12, 0x49, 0x3A},
	{0x0F, 0x12, 0x68, 0x49},
	{0x0F, 0x12, 0x42, 0x81},
	{0x0F, 0x12, 0xD8, 0x00},
	{0x0F, 0x12, 0x1C, 0x08},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xAF},
	{0x0F, 0x12, 0x60, 0x20},
	{0x0F, 0x12, 0xE7, 0xDC},
	{0x0F, 0x12, 0xB5, 0x70},
	{0x0F, 0x12, 0x68, 0x01},
	{0x0F, 0x12, 0x04, 0x0D},
	{0x0F, 0x12, 0x0C, 0x2D},
	{0x0F, 0x12, 0x68, 0x44},
	{0x0F, 0x12, 0x48, 0x33},
	{0x0F, 0x12, 0x89, 0x81},
	{0x0F, 0x12, 0x1C, 0x28},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0x93},
	{0x0F, 0x12, 0x80, 0x60},
	{0x0F, 0x12, 0x49, 0x32},
	{0x0F, 0x12, 0x69, 0xC9},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xA6},
	{0x0F, 0x12, 0x1C, 0x01},
	{0x0F, 0x12, 0x80, 0xA0},
	{0x0F, 0x12, 0x02, 0x28},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0xA9},
	{0x0F, 0x12, 0x04, 0x00},
	{0x0F, 0x12, 0x0C, 0x00},
	{0x0F, 0x12, 0x80, 0x20},
	{0x0F, 0x12, 0x49, 0x2D},
	{0x0F, 0x12, 0x23, 0x00},
	{0x0F, 0x12, 0x5E, 0xC9},
	{0x0F, 0x12, 0x42, 0x88},
	{0x0F, 0x12, 0xDA, 0x02},
	{0x0F, 0x12, 0x20, 0xFF},
	{0x0F, 0x12, 0x30, 0x01},
	{0x0F, 0x12, 0x80, 0x20},
	{0x0F, 0x12, 0xBC, 0x70},
	{0x0F, 0x12, 0xBC, 0x08},
	{0x0F, 0x12, 0x47, 0x18},
	{0x0F, 0x12, 0xB5, 0x70},
	{0x0F, 0x12, 0x1C, 0x04},
	{0x0F, 0x12, 0x48, 0x28},
	{0x0F, 0x12, 0x49, 0x26},
	{0x0F, 0x12, 0x78, 0x03},
	{0x0F, 0x12, 0x6A, 0x8A},
	{0x0F, 0x12, 0x2B, 0x00},
	{0x0F, 0x12, 0xD1, 0x00},
	{0x0F, 0x12, 0x6A, 0x0A},
	{0x0F, 0x12, 0x4D, 0x20},
	{0x0F, 0x12, 0x2B, 0x00},
	{0x0F, 0x12, 0x68, 0xA8},
	{0x0F, 0x12, 0xD1, 0x00},
	{0x0F, 0x12, 0x68, 0x68},
	{0x0F, 0x12, 0x68, 0x23},
	{0x0F, 0x12, 0x8D, 0xC9},
	{0x0F, 0x12, 0x43, 0x4A},
	{0x0F, 0x12, 0x0A, 0x12},
	{0x0F, 0x12, 0x42, 0x9A},
	{0x0F, 0x12, 0xD3, 0x0D},
	{0x0F, 0x12, 0x4D, 0x20},
	{0x0F, 0x12, 0x26, 0xFF},
	{0x0F, 0x12, 0x88, 0x28},
	{0x0F, 0x12, 0x36, 0x01},
	{0x0F, 0x12, 0x43, 0xB0},
	{0x0F, 0x12, 0x80, 0x28},
	{0x0F, 0x12, 0x68, 0x20},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0x84},
	{0x0F, 0x12, 0x60, 0x20},
	{0x0F, 0x12, 0x88, 0x28},
	{0x0F, 0x12, 0x43, 0x30},
	{0x0F, 0x12, 0x80, 0x28},
	{0x0F, 0x12, 0xE7, 0xDA},
	{0x0F, 0x12, 0x1C, 0x0A},
	{0x0F, 0x12, 0x43, 0x42},
	{0x0F, 0x12, 0x0A, 0x12},
	{0x0F, 0x12, 0x42, 0x9A},
	{0x0F, 0x12, 0xD3, 0x04},
	{0x0F, 0x12, 0x02, 0x18},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0x71},
	{0x0F, 0x12, 0x60, 0x20},
	{0x0F, 0x12, 0xE7, 0xF4},
	{0x0F, 0x12, 0x60, 0x20},
	{0x0F, 0x12, 0xE7, 0xF2},
	{0x0F, 0x12, 0xB5, 0x10},
	{0x0F, 0x12, 0x49, 0x13},
	{0x0F, 0x12, 0x8F, 0xC8},
	{0x0F, 0x12, 0x28, 0x00},
	{0x0F, 0x12, 0xD0, 0x07},
	{0x0F, 0x12, 0x20, 0x00},
	{0x0F, 0x12, 0x87, 0xC8},
	{0x0F, 0x12, 0x8F, 0x88},
	{0x0F, 0x12, 0x4C, 0x11},
	{0x0F, 0x12, 0x28, 0x00},
	{0x0F, 0x12, 0xD0, 0x02},
	{0x0F, 0x12, 0x20, 0x08},
	{0x0F, 0x12, 0x80, 0x20},
	{0x0F, 0x12, 0xE7, 0x7E},
	{0x0F, 0x12, 0x48, 0x0D},
	{0x0F, 0x12, 0x30, 0x60},
	{0x0F, 0x12, 0x89, 0x00},
	{0x0F, 0x12, 0x28, 0x00},
	{0x0F, 0x12, 0xD1, 0x03},
	{0x0F, 0x12, 0x48, 0x0C},
	{0x0F, 0x12, 0x21, 0x01},
	{0x0F, 0x12, 0xF0, 0x00},
	{0x0F, 0x12, 0xF8, 0x64},
	{0x0F, 0x12, 0x20, 0x10},
	{0x0F, 0x12, 0x80, 0x20},
	{0x0F, 0x12, 0xE7, 0xF2},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0xF4, 0xB0},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x20, 0x58},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x15, 0x54},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x00, 0x80},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x04, 0x6C},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x04, 0x68},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x11, 0x00},
	{0x0F, 0x12, 0xD0, 0x00},
	{0x0F, 0x12, 0x01, 0xB8},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x04, 0x4E},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x04, 0x50},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xC0, 0x00},
	{0x0F, 0x12, 0xE5, 0x9F},
	{0x0F, 0x12, 0xFF, 0x1C},
	{0x0F, 0x12, 0xE1, 0x2F},
	{0x0F, 0x12, 0x9C, 0xE7},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xF0, 0x04},
	{0x0F, 0x12, 0xE5, 0x1F},
	{0x0F, 0x12, 0x9F, 0xB8},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xC0, 0x00},
	{0x0F, 0x12, 0xE5, 0x9F},
	{0x0F, 0x12, 0xFF, 0x1C},
	{0x0F, 0x12, 0xE1, 0x2F},
	{0x0F, 0x12, 0x88, 0xDF},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xC0, 0x00},
	{0x0F, 0x12, 0xE5, 0x9F},
	{0x0F, 0x12, 0xFF, 0x1C},
	{0x0F, 0x12, 0xE1, 0x2F},
	{0x0F, 0x12, 0x27, 0x5D},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xC0, 0x00},
	{0x0F, 0x12, 0xE5, 0x9F},
	{0x0F, 0x12, 0xFF, 0x1C},
	{0x0F, 0x12, 0xE1, 0x2F},
	{0x0F, 0x12, 0x1E, 0xD3},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xC0, 0x00},
	{0x0F, 0x12, 0xE5, 0x9F},
	{0x0F, 0x12, 0xFF, 0x1C},
	{0x0F, 0x12, 0xE1, 0x2F},
	{0x0F, 0x12, 0x26, 0xF9},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xC0, 0x00},
	{0x0F, 0x12, 0xE5, 0x9F},
	{0x0F, 0x12, 0xFF, 0x1C},
	{0x0F, 0x12, 0xE1, 0x2F},
	{0x0F, 0x12, 0x40, 0x27},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xC0, 0x00},
	{0x0F, 0x12, 0xE5, 0x9F},
	{0x0F, 0x12, 0xFF, 0x1C},
	{0x0F, 0x12, 0xE1, 0x2F},
	{0x0F, 0x12, 0x9F, 0x03},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xF0, 0x04},
	{0x0F, 0x12, 0xE5, 0x1F},
	{0x0F, 0x12, 0xA1, 0x44},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xC0, 0x00},
	{0x0F, 0x12, 0xE5, 0x9F},
	{0x0F, 0x12, 0xFF, 0x1C},
	{0x0F, 0x12, 0xE1, 0x2F},
	{0x0F, 0x12, 0x28, 0x5F},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x47, 0x78},
	{0x0F, 0x12, 0x46, 0xC0},
	{0x0F, 0x12, 0xC0, 0x00},
	{0x0F, 0x12, 0xE5, 0x9F},
	{0x0F, 0x12, 0xFF, 0x1C},
	{0x0F, 0x12, 0xE1, 0x2F},
	{0x0F, 0x12, 0x20, 0x01},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0xE8, 0x48},
	{0x0F, 0x12, 0x00, 0x01},
	{0x0F, 0x12, 0xE8, 0x48},
	{0x0F, 0x12, 0x00, 0x01},
	{0x0F, 0x12, 0x05, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	/* End T&P part */

	/* Start tuning part */

	/*
	 * Write memory example:
	 * m70003000 10CE
	 * Write HW register example:
	 * 1004 5678
	 */

	{0xF4, 0x54, 0x00, 0x01},

	{0x00, 0x2A, 0x0E, 0x3A},
	{0x0F, 0x12, 0x02, 0xC2},

	/* Analog Settings */
	{0x00, 0x2A, 0x11, 0x2A},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x11, 0x32},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x11, 0x3E},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x11, 0x5C},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x11, 0x64},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x11, 0x74},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x11, 0x78},
	{0x0F, 0x12, 0x00, 0x00},

	{0x00, 0x2A, 0x07, 0x7A},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},

	{0x00, 0x2A, 0x07, 0xA2},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},

	{0x00, 0x2A, 0x07, 0xB6},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x02},
	{0x0F, 0x12, 0x00, 0x04},
	{0x0F, 0x12, 0x00, 0x04},
	{0x0F, 0x12, 0x00, 0x05},
	{0x0F, 0x12, 0x00, 0x05},

	/*	param_start	TVAR_ash_GASAlpha */
	{0x00, 0x2A, 0x07, 0x12},
	{0x0F, 0x12, 0x01, 0x76},	/* TVAR_ash_GASAlpha[0] */
	{0x0F, 0x12, 0x01, 0x45},	/* TVAR_ash_GASAlpha[1] */
	{0x0F, 0x12, 0x01, 0x47},	/* TVAR_ash_GASAlpha[2] */
	{0x0F, 0x12, 0x01, 0x11},	/* TVAR_ash_GASAlpha[3] */
	{0x0F, 0x12, 0x01, 0x45},	/* TVAR_ash_GASAlpha[4] */
	{0x0F, 0x12, 0x01, 0x22},	/* TVAR_ash_GASAlpha[5] */
	{0x0F, 0x12, 0x01, 0x22},	/* TVAR_ash_GASAlpha[6] */
	{0x0F, 0x12, 0x01, 0x03},	/* TVAR_ash_GASAlpha[7] */
	{0x0F, 0x12, 0x01, 0x12},	/* TVAR_ash_GASAlpha[8] */
	{0x0F, 0x12, 0x00, 0xEE},	/* TVAR_ash_GASAlpha[9] */
	{0x0F, 0x12, 0x00, 0xF2},	/* TVAR_ash_GASAlpha[10] */
	{0x0F, 0x12, 0x00, 0xD7},	/* TVAR_ash_GASAlpha[11] */
	{0x0F, 0x12, 0x00, 0xEF},	/* TVAR_ash_GASAlpha[12] */
	{0x0F, 0x12, 0x01, 0x0C},	/* TVAR_ash_GASAlpha[13] */
	{0x0F, 0x12, 0x01, 0x0F},	/* TVAR_ash_GASAlpha[14] */
	{0x0F, 0x12, 0x00, 0xF8},	/* TVAR_ash_GASAlpha[15] */
	{0x0F, 0x12, 0x01, 0x1C},	/* TVAR_ash_GASAlpha[16] */
	{0x0F, 0x12, 0x01, 0x1F},	/* TVAR_ash_GASAlpha[17] */
	{0x0F, 0x12, 0x01, 0x1E},	/* TVAR_ash_GASAlpha[18] */
	{0x0F, 0x12, 0x01, 0x27},	/* TVAR_ash_GASAlpha[19] */
	{0x0F, 0x12, 0x00, 0xF6},	/* TVAR_ash_GASAlpha[20] */
	{0x0F, 0x12, 0x00, 0xF7},	/* TVAR_ash_GASAlpha[21] */
	{0x0F, 0x12, 0x00, 0xF5},	/* TVAR_ash_GASAlpha[22] */
	{0x0F, 0x12, 0x01, 0x04},	/* TVAR_ash_GASAlpha[23] */
	{0x0F, 0x12, 0x00, 0xE9},	/* TVAR_ash_GASAlpha[24] */
	{0x0F, 0x12, 0x00, 0xF1},	/* TVAR_ash_GASAlpha[25] */
	{0x0F, 0x12, 0x00, 0xEE},	/* TVAR_ash_GASAlpha[26] */
	{0x0F, 0x12, 0x00, 0xFE},	/* TVAR_ash_GASAlpha[27] */
	/* param_end	TVAR_ash_GASAlpha */

	/* param_start	TVAR_ash_GASOutdoorAlpha */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_ash_GASOutdoorAlpha[0] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_ash_GASOutdoorAlpha[1] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_ash_GASOutdoorAlpha[2] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_ash_GASOutdoorAlpha[3] */
	/* param_end	TVAR_ash_GASOutdoorAlpha */

	{0x00, 0x2A, 0x07, 0x5A},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x02, 0x80},
	{0x0F, 0x12, 0x02, 0x00},
	{0x0F, 0x12, 0x00, 0x0E},
	{0x0F, 0x12, 0x00, 0x0F},


	/* param_start	TVAR_ash_pGAS */
	{0x00, 0x2A, 0x24, 0x7C},
	{0x0F, 0x12, 0x01, 0x7E},	/* TVAR_ash_pGAS[0] */
	{0x0F, 0x12, 0x01, 0x2A},	/* TVAR_ash_pGAS[1] */
	{0x0F, 0x12, 0x00, 0xF0},	/* TVAR_ash_pGAS[2] */
	{0x0F, 0x12, 0x00, 0xCB},	/* TVAR_ash_pGAS[3] */
	{0x0F, 0x12, 0x00, 0xB3},	/* TVAR_ash_pGAS[4] */
	{0x0F, 0x12, 0x00, 0xA4},	/* TVAR_ash_pGAS[5] */
	{0x0F, 0x12, 0x00, 0x9D},	/* TVAR_ash_pGAS[6] */
	{0x0F, 0x12, 0x00, 0xA1},	/* TVAR_ash_pGAS[7] */
	{0x0F, 0x12, 0x00, 0xAD},	/* TVAR_ash_pGAS[8] */
	{0x0F, 0x12, 0x00, 0xBF},	/* TVAR_ash_pGAS[9] */
	{0x0F, 0x12, 0x00, 0xDE},	/* TVAR_ash_pGAS[10] */
	{0x0F, 0x12, 0x01, 0x0F},	/* TVAR_ash_pGAS[11] */
	{0x0F, 0x12, 0x01, 0x4C},	/* TVAR_ash_pGAS[12] */
	{0x0F, 0x12, 0x01, 0x39},	/* TVAR_ash_pGAS[13] */
	{0x0F, 0x12, 0x00, 0xFA},	/* TVAR_ash_pGAS[14] */
	{0x0F, 0x12, 0x00, 0xC4},	/* TVAR_ash_pGAS[15] */
	{0x0F, 0x12, 0x00, 0xA3},	/* TVAR_ash_pGAS[16] */
	{0x0F, 0x12, 0x00, 0x8A},	/* TVAR_ash_pGAS[17] */
	{0x0F, 0x12, 0x00, 0x7A},	/* TVAR_ash_pGAS[18] */
	{0x0F, 0x12, 0x00, 0x71},	/* TVAR_ash_pGAS[19] */
	{0x0F, 0x12, 0x00, 0x74},	/* TVAR_ash_pGAS[20] */
	{0x0F, 0x12, 0x00, 0x82},	/* TVAR_ash_pGAS[21] */
	{0x0F, 0x12, 0x00, 0x96},	/* TVAR_ash_pGAS[22] */
	{0x0F, 0x12, 0x00, 0xB4},	/* TVAR_ash_pGAS[23] */
	{0x0F, 0x12, 0x00, 0xE6},	/* TVAR_ash_pGAS[24] */
	{0x0F, 0x12, 0x01, 0x1B},	/* TVAR_ash_pGAS[25] */
	{0x0F, 0x12, 0x01, 0x0F},	/* TVAR_ash_pGAS[26] */
	{0x0F, 0x12, 0x00, 0xD1},	/* TVAR_ash_pGAS[27] */
	{0x0F, 0x12, 0x00, 0xA3},	/* TVAR_ash_pGAS[28] */
	{0x0F, 0x12, 0x00, 0x7E},	/* TVAR_ash_pGAS[29] */
	{0x0F, 0x12, 0x00, 0x60},	/* TVAR_ash_pGAS[30] */
	{0x0F, 0x12, 0x00, 0x4E},	/* TVAR_ash_pGAS[31] */
	{0x0F, 0x12, 0x00, 0x49},	/* TVAR_ash_pGAS[32] */
	{0x0F, 0x12, 0x00, 0x46},	/* TVAR_ash_pGAS[33] */
	{0x0F, 0x12, 0x00, 0x52},	/* TVAR_ash_pGAS[34] */
	{0x0F, 0x12, 0x00, 0x6B},	/* TVAR_ash_pGAS[35] */
	{0x0F, 0x12, 0x00, 0x8B},	/* TVAR_ash_pGAS[36] */
	{0x0F, 0x12, 0x00, 0xBA},	/* TVAR_ash_pGAS[37] */
	{0x0F, 0x12, 0x00, 0xF1},	/* TVAR_ash_pGAS[38] */
	{0x0F, 0x12, 0x00, 0xF0},	/* TVAR_ash_pGAS[39] */
	{0x0F, 0x12, 0x00, 0xB6},	/* TVAR_ash_pGAS[40] */
	{0x0F, 0x12, 0x00, 0x88},	/* TVAR_ash_pGAS[41] */
	{0x0F, 0x12, 0x00, 0x60},	/* TVAR_ash_pGAS[42] */
	{0x0F, 0x12, 0x00, 0x42},	/* TVAR_ash_pGAS[43] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_ash_pGAS[44] */
	{0x0F, 0x12, 0x00, 0x21},	/* TVAR_ash_pGAS[45] */
	{0x0F, 0x12, 0x00, 0x24},	/* TVAR_ash_pGAS[46] */
	{0x0F, 0x12, 0x00, 0x30},	/* TVAR_ash_pGAS[47] */
	{0x0F, 0x12, 0x00, 0x4C},	/* TVAR_ash_pGAS[48] */
	{0x0F, 0x12, 0x00, 0x6E},	/* TVAR_ash_pGAS[49] */
	{0x0F, 0x12, 0x00, 0xA1},	/* TVAR_ash_pGAS[50] */
	{0x0F, 0x12, 0x00, 0xCF},	/* TVAR_ash_pGAS[51] */
	{0x0F, 0x12, 0x00, 0xDB},	/* TVAR_ash_pGAS[52] */
	{0x0F, 0x12, 0x00, 0xA8},	/* TVAR_ash_pGAS[53] */
	{0x0F, 0x12, 0x00, 0x7D},	/* TVAR_ash_pGAS[54] */
	{0x0F, 0x12, 0x00, 0x4F},	/* TVAR_ash_pGAS[55] */
	{0x0F, 0x12, 0x00, 0x2E},	/* TVAR_ash_pGAS[56] */
	{0x0F, 0x12, 0x00, 0x11},	/* TVAR_ash_pGAS[57] */
	{0x0F, 0x12, 0x00, 0x07},	/* TVAR_ash_pGAS[58] */
	{0x0F, 0x12, 0x00, 0x0C},	/* TVAR_ash_pGAS[59] */
	{0x0F, 0x12, 0x00, 0x1A},	/* TVAR_ash_pGAS[60] */
	{0x0F, 0x12, 0x00, 0x38},	/* TVAR_ash_pGAS[61] */
	{0x0F, 0x12, 0x00, 0x5E},	/* TVAR_ash_pGAS[62] */
	{0x0F, 0x12, 0x00, 0x8B},	/* TVAR_ash_pGAS[63] */
	{0x0F, 0x12, 0x00, 0xC5},	/* TVAR_ash_pGAS[64] */
	{0x0F, 0x12, 0x00, 0xD9},	/* TVAR_ash_pGAS[65] */
	{0x0F, 0x12, 0x00, 0xA5},	/* TVAR_ash_pGAS[66] */
	{0x0F, 0x12, 0x00, 0x77},	/* TVAR_ash_pGAS[67] */
	{0x0F, 0x12, 0x00, 0x4C},	/* TVAR_ash_pGAS[68] */
	{0x0F, 0x12, 0x00, 0x25},	/* TVAR_ash_pGAS[69] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_ash_pGAS[70] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_ash_pGAS[71] */
	{0x0F, 0x12, 0x00, 0x02},	/* TVAR_ash_pGAS[72] */
	{0x0F, 0x12, 0x00, 0x12},	/* TVAR_ash_pGAS[73] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_ash_pGAS[74] */
	{0x0F, 0x12, 0x00, 0x5A},	/* TVAR_ash_pGAS[75] */
	{0x0F, 0x12, 0x00, 0x8C},	/* TVAR_ash_pGAS[76] */
	{0x0F, 0x12, 0x00, 0xC3},	/* TVAR_ash_pGAS[77] */
	{0x0F, 0x12, 0x00, 0xE4},	/* TVAR_ash_pGAS[78] */
	{0x0F, 0x12, 0x00, 0xAE},	/* TVAR_ash_pGAS[79] */
	{0x0F, 0x12, 0x00, 0x7E},	/* TVAR_ash_pGAS[80] */
	{0x0F, 0x12, 0x00, 0x52},	/* TVAR_ash_pGAS[81] */
	{0x0F, 0x12, 0x00, 0x2C},	/* TVAR_ash_pGAS[82] */
	{0x0F, 0x12, 0x00, 0x14},	/* TVAR_ash_pGAS[83] */
	{0x0F, 0x12, 0x00, 0x07},	/* TVAR_ash_pGAS[84] */
	{0x0F, 0x12, 0x00, 0x0B},	/* TVAR_ash_pGAS[85] */
	{0x0F, 0x12, 0x00, 0x1C},	/* TVAR_ash_pGAS[86] */
	{0x0F, 0x12, 0x00, 0x3A},	/* TVAR_ash_pGAS[87] */
	{0x0F, 0x12, 0x00, 0x62},	/* TVAR_ash_pGAS[88] */
	{0x0F, 0x12, 0x00, 0x96},	/* TVAR_ash_pGAS[89] */
	{0x0F, 0x12, 0x00, 0xCC},	/* TVAR_ash_pGAS[90] */
	{0x0F, 0x12, 0x00, 0xFC},	/* TVAR_ash_pGAS[91] */
	{0x0F, 0x12, 0x00, 0xC2},	/* TVAR_ash_pGAS[92] */
	{0x0F, 0x12, 0x00, 0x8F},	/* TVAR_ash_pGAS[93] */
	{0x0F, 0x12, 0x00, 0x67},	/* TVAR_ash_pGAS[94] */
	{0x0F, 0x12, 0x00, 0x44},	/* TVAR_ash_pGAS[95] */
	{0x0F, 0x12, 0x00, 0x2D},	/* TVAR_ash_pGAS[96] */
	{0x0F, 0x12, 0x00, 0x1F},	/* TVAR_ash_pGAS[97] */
	{0x0F, 0x12, 0x00, 0x24},	/* TVAR_ash_pGAS[98] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_ash_pGAS[99] */
	{0x0F, 0x12, 0x00, 0x51},	/* TVAR_ash_pGAS[100] */
	{0x0F, 0x12, 0x00, 0x78},	/* TVAR_ash_pGAS[101] */
	{0x0F, 0x12, 0x00, 0xAD},	/* TVAR_ash_pGAS[102] */
	{0x0F, 0x12, 0x00, 0xE5},	/* TVAR_ash_pGAS[103] */
	{0x0F, 0x12, 0x01, 0x27},	/* TVAR_ash_pGAS[104] */
	{0x0F, 0x12, 0x00, 0xE7},	/* TVAR_ash_pGAS[105] */
	{0x0F, 0x12, 0x00, 0xAE},	/* TVAR_ash_pGAS[106] */
	{0x0F, 0x12, 0x00, 0x88},	/* TVAR_ash_pGAS[107] */
	{0x0F, 0x12, 0x00, 0x66},	/* TVAR_ash_pGAS[108] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_ash_pGAS[109] */
	{0x0F, 0x12, 0x00, 0x45},	/* TVAR_ash_pGAS[110] */
	{0x0F, 0x12, 0x00, 0x48},	/* TVAR_ash_pGAS[111] */
	{0x0F, 0x12, 0x00, 0x57},	/* TVAR_ash_pGAS[112] */
	{0x0F, 0x12, 0x00, 0x74},	/* TVAR_ash_pGAS[113] */
	{0x0F, 0x12, 0x00, 0x9B},	/* TVAR_ash_pGAS[114] */
	{0x0F, 0x12, 0x00, 0xCF},	/* TVAR_ash_pGAS[115] */
	{0x0F, 0x12, 0x01, 0x0D},	/* TVAR_ash_pGAS[116] */
	{0x0F, 0x12, 0x01, 0x6C},	/* TVAR_ash_pGAS[117] */
	{0x0F, 0x12, 0x01, 0x1D},	/* TVAR_ash_pGAS[118] */
	{0x0F, 0x12, 0x00, 0xDD},	/* TVAR_ash_pGAS[119] */
	{0x0F, 0x12, 0x00, 0xB6},	/* TVAR_ash_pGAS[120] */
	{0x0F, 0x12, 0x00, 0x97},	/* TVAR_ash_pGAS[121] */
	{0x0F, 0x12, 0x00, 0x82},	/* TVAR_ash_pGAS[122] */
	{0x0F, 0x12, 0x00, 0x77},	/* TVAR_ash_pGAS[123] */
	{0x0F, 0x12, 0x00, 0x7C},	/* TVAR_ash_pGAS[124] */
	{0x0F, 0x12, 0x00, 0x8B},	/* TVAR_ash_pGAS[125] */
	{0x0F, 0x12, 0x00, 0xA2},	/* TVAR_ash_pGAS[126] */
	{0x0F, 0x12, 0x00, 0xCA},	/* TVAR_ash_pGAS[127] */
	{0x0F, 0x12, 0x01, 0x05},	/* TVAR_ash_pGAS[128] */
	{0x0F, 0x12, 0x01, 0x4A},	/* TVAR_ash_pGAS[129] */
	{0x0F, 0x12, 0x01, 0xAD},	/* TVAR_ash_pGAS[130] */
	{0x0F, 0x12, 0x01, 0x48},	/* TVAR_ash_pGAS[131] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_ash_pGAS[132] */
	{0x0F, 0x12, 0x00, 0xD2},	/* TVAR_ash_pGAS[133] */
	{0x0F, 0x12, 0x00, 0xB6},	/* TVAR_ash_pGAS[134] */
	{0x0F, 0x12, 0x00, 0xA5},	/* TVAR_ash_pGAS[135] */
	{0x0F, 0x12, 0x00, 0x9B},	/* TVAR_ash_pGAS[136] */
	{0x0F, 0x12, 0x00, 0xA3},	/* TVAR_ash_pGAS[137] */
	{0x0F, 0x12, 0x00, 0xB0},	/* TVAR_ash_pGAS[138] */
	{0x0F, 0x12, 0x00, 0xCA},	/* TVAR_ash_pGAS[139] */
	{0x0F, 0x12, 0x00, 0xF7},	/* TVAR_ash_pGAS[140] */
	{0x0F, 0x12, 0x01, 0x3D},	/* TVAR_ash_pGAS[141] */
	{0x0F, 0x12, 0x01, 0x8C},	/* TVAR_ash_pGAS[142] */
	{0x0F, 0x12, 0x01, 0x48},	/* TVAR_ash_pGAS[143] */
	{0x0F, 0x12, 0x00, 0xF3},	/* TVAR_ash_pGAS[144] */
	{0x0F, 0x12, 0x00, 0xC2},	/* TVAR_ash_pGAS[145] */
	{0x0F, 0x12, 0x00, 0xA0},	/* TVAR_ash_pGAS[146] */
	{0x0F, 0x12, 0x00, 0x8C},	/* TVAR_ash_pGAS[147] */
	{0x0F, 0x12, 0x00, 0x80},	/* TVAR_ash_pGAS[148] */
	{0x0F, 0x12, 0x00, 0x7A},	/* TVAR_ash_pGAS[149] */
	{0x0F, 0x12, 0x00, 0x7A},	/* TVAR_ash_pGAS[150] */
	{0x0F, 0x12, 0x00, 0x80},	/* TVAR_ash_pGAS[151] */
	{0x0F, 0x12, 0x00, 0x89},	/* TVAR_ash_pGAS[152] */
	{0x0F, 0x12, 0x00, 0xA2},	/* TVAR_ash_pGAS[153] */
	{0x0F, 0x12, 0x00, 0xC8},	/* TVAR_ash_pGAS[154] */
	{0x0F, 0x12, 0x01, 0x02},	/* TVAR_ash_pGAS[155] */
	{0x0F, 0x12, 0x01, 0x07},	/* TVAR_ash_pGAS[156] */
	{0x0F, 0x12, 0x00, 0xCA},	/* TVAR_ash_pGAS[157] */
	{0x0F, 0x12, 0x00, 0x9C},	/* TVAR_ash_pGAS[158] */
	{0x0F, 0x12, 0x00, 0x81},	/* TVAR_ash_pGAS[159] */
	{0x0F, 0x12, 0x00, 0x6D},	/* TVAR_ash_pGAS[160] */
	{0x0F, 0x12, 0x00, 0x62},	/* TVAR_ash_pGAS[161] */
	{0x0F, 0x12, 0x00, 0x59},	/* TVAR_ash_pGAS[162] */
	{0x0F, 0x12, 0x00, 0x59},	/* TVAR_ash_pGAS[163] */
	{0x0F, 0x12, 0x00, 0x5F},	/* TVAR_ash_pGAS[164] */
	{0x0F, 0x12, 0x00, 0x69},	/* TVAR_ash_pGAS[165] */
	{0x0F, 0x12, 0x00, 0x7D},	/* TVAR_ash_pGAS[166] */
	{0x0F, 0x12, 0x00, 0xA3},	/* TVAR_ash_pGAS[167] */
	{0x0F, 0x12, 0x00, 0xD2},	/* TVAR_ash_pGAS[168] */
	{0x0F, 0x12, 0x00, 0xE2},	/* TVAR_ash_pGAS[169] */
	{0x0F, 0x12, 0x00, 0xA7},	/* TVAR_ash_pGAS[170] */
	{0x0F, 0x12, 0x00, 0x82},	/* TVAR_ash_pGAS[171] */
	{0x0F, 0x12, 0x00, 0x66},	/* TVAR_ash_pGAS[172] */
	{0x0F, 0x12, 0x00, 0x4F},	/* TVAR_ash_pGAS[173] */
	{0x0F, 0x12, 0x00, 0x41},	/* TVAR_ash_pGAS[174] */
	{0x0F, 0x12, 0x00, 0x3D},	/* TVAR_ash_pGAS[175] */
	{0x0F, 0x12, 0x00, 0x37},	/* TVAR_ash_pGAS[176] */
	{0x0F, 0x12, 0x00, 0x3B},	/* TVAR_ash_pGAS[177] */
	{0x0F, 0x12, 0x00, 0x4A},	/* TVAR_ash_pGAS[178] */
	{0x0F, 0x12, 0x00, 0x5E},	/* TVAR_ash_pGAS[179] */
	{0x0F, 0x12, 0x00, 0x7E},	/* TVAR_ash_pGAS[180] */
	{0x0F, 0x12, 0x00, 0xAB},	/* TVAR_ash_pGAS[181] */
	{0x0F, 0x12, 0x00, 0xC3},	/* TVAR_ash_pGAS[182] */
	{0x0F, 0x12, 0x00, 0x8E},	/* TVAR_ash_pGAS[183] */
	{0x0F, 0x12, 0x00, 0x6C},	/* TVAR_ash_pGAS[184] */
	{0x0F, 0x12, 0x00, 0x4E},	/* TVAR_ash_pGAS[185] */
	{0x0F, 0x12, 0x00, 0x37},	/* TVAR_ash_pGAS[186] */
	{0x0F, 0x12, 0x00, 0x22},	/* TVAR_ash_pGAS[187] */
	{0x0F, 0x12, 0x00, 0x1C},	/* TVAR_ash_pGAS[188] */
	{0x0F, 0x12, 0x00, 0x1D},	/* TVAR_ash_pGAS[189] */
	{0x0F, 0x12, 0x00, 0x22},	/* TVAR_ash_pGAS[190] */
	{0x0F, 0x12, 0x00, 0x33},	/* TVAR_ash_pGAS[191] */
	{0x0F, 0x12, 0x00, 0x49},	/* TVAR_ash_pGAS[192] */
	{0x0F, 0x12, 0x00, 0x6B},	/* TVAR_ash_pGAS[193] */
	{0x0F, 0x12, 0x00, 0x8E},	/* TVAR_ash_pGAS[194] */
	{0x0F, 0x12, 0x00, 0xB0},	/* TVAR_ash_pGAS[195] */
	{0x0F, 0x12, 0x00, 0x83},	/* TVAR_ash_pGAS[196] */
	{0x0F, 0x12, 0x00, 0x62},	/* TVAR_ash_pGAS[197] */
	{0x0F, 0x12, 0x00, 0x40},	/* TVAR_ash_pGAS[198] */
	{0x0F, 0x12, 0x00, 0x27},	/* TVAR_ash_pGAS[199] */
	{0x0F, 0x12, 0x00, 0x0F},	/* TVAR_ash_pGAS[200] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_ash_pGAS[201] */
	{0x0F, 0x12, 0x00, 0x08},	/* TVAR_ash_pGAS[202] */
	{0x0F, 0x12, 0x00, 0x10},	/* TVAR_ash_pGAS[203] */
	{0x0F, 0x12, 0x00, 0x23},	/* TVAR_ash_pGAS[204] */
	{0x0F, 0x12, 0x00, 0x3D},	/* TVAR_ash_pGAS[205] */
	{0x0F, 0x12, 0x00, 0x5A},	/* TVAR_ash_pGAS[206] */
	{0x0F, 0x12, 0x00, 0x86},	/* TVAR_ash_pGAS[207] */
	{0x0F, 0x12, 0x00, 0xAE},	/* TVAR_ash_pGAS[208] */
	{0x0F, 0x12, 0x00, 0x80},	/* TVAR_ash_pGAS[209] */
	{0x0F, 0x12, 0x00, 0x5E},	/* TVAR_ash_pGAS[210] */
	{0x0F, 0x12, 0x00, 0x3E},	/* TVAR_ash_pGAS[211] */
	{0x0F, 0x12, 0x00, 0x1F},	/* TVAR_ash_pGAS[212] */
	{0x0F, 0x12, 0x0, 0x009},	/* TVAR_ash_pGAS[213] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_ash_pGAS[214] */
	{0x0F, 0x12, 0x00, 0x01},	/* TVAR_ash_pGAS[215] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_ash_pGAS[216] */
	{0x0F, 0x12, 0x00, 0x1F},	/* TVAR_ash_pGAS[217] */
	{0x0F, 0x12, 0x00, 0x3B},	/* TVAR_ash_pGAS[218] */
	{0x0F, 0x12, 0x00, 0x5C},	/* TVAR_ash_pGAS[219] */
	{0x0F, 0x12, 0x00, 0x85},	/* TVAR_ash_pGAS[220] */
	{0x0F, 0x12, 0x00, 0xB6},	/* TVAR_ash_pGAS[221] */
	{0x0F, 0x12, 0x00, 0x87},	/* TVAR_ash_pGAS[222] */
	{0x0F, 0x12, 0x00, 0x64},	/* TVAR_ash_pGAS[223] */
	{0x0F, 0x12, 0x00, 0x42},	/* TVAR_ash_pGAS[224] */
	{0x0F, 0x12, 0x00, 0x25},	/* TVAR_ash_pGAS[225] */
	{0x0F, 0x12, 0x00, 0x11},	/* TVAR_ash_pGAS[226] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_ash_pGAS[227] */
	{0x0F, 0x12, 0x00, 0x08},	/* TVAR_ash_pGAS[228] */
	{0x0F, 0x12, 0x00, 0x12},	/* TVAR_ash_pGAS[229] */
	{0x0F, 0x12, 0x00, 0x26},	/* TVAR_ash_pGAS[230] */
	{0x0F, 0x12, 0x00, 0x41},	/* TVAR_ash_pGAS[231] */
	{0x0F, 0x12, 0x00, 0x64},	/* TVAR_ash_pGAS[232] */
	{0x0F, 0x12, 0x00, 0x8D},	/* TVAR_ash_pGAS[233] */
	{0x0F, 0x12, 0x00, 0xCA},	/* TVAR_ash_pGAS[234] */
	{0x0F, 0x12, 0x00, 0x97},	/* TVAR_ash_pGAS[235] */
	{0x0F, 0x12, 0x00, 0x71},	/* TVAR_ash_pGAS[236] */
	{0x0F, 0x12, 0x00, 0x52},	/* TVAR_ash_pGAS[237] */
	{0x0F, 0x12, 0x00, 0x37},	/* TVAR_ash_pGAS[238] */
	{0x0F, 0x12, 0x00, 0x26},	/* TVAR_ash_pGAS[239] */
	{0x0F, 0x12, 0x00, 0x1A},	/* TVAR_ash_pGAS[240] */
	{0x0F, 0x12, 0x00, 0x1C},	/* TVAR_ash_pGAS[241] */
	{0x0F, 0x12, 0x00, 0x24},	/* TVAR_ash_pGAS[242] */
	{0x0F, 0x12, 0x00, 0x39},	/* TVAR_ash_pGAS[243] */
	{0x0F, 0x12, 0x00, 0x53},	/* TVAR_ash_pGAS[244] */
	{0x0F, 0x12, 0x00, 0x76},	/* TVAR_ash_pGAS[245] */
	{0x0F, 0x12, 0x00, 0xA2},	/* TVAR_ash_pGAS[246] */
	{0x0F, 0x12, 0x00, 0xF0},	/* TVAR_ash_pGAS[247] */
	{0x0F, 0x12, 0x00, 0xB6},	/* TVAR_ash_pGAS[248] */
	{0x0F, 0x12, 0x00, 0x89},	/* TVAR_ash_pGAS[249] */
	{0x0F, 0x12, 0x00, 0x6B},	/* TVAR_ash_pGAS[250] */
	{0x0F, 0x12, 0x00, 0x52},	/* TVAR_ash_pGAS[251] */
	{0x0F, 0x12, 0x00, 0x41},	/* TVAR_ash_pGAS[252] */
	{0x0F, 0x12, 0x00, 0x38},	/* TVAR_ash_pGAS[253] */
	{0x0F, 0x12, 0x00, 0x39},	/* TVAR_ash_pGAS[254] */
	{0x0F, 0x12, 0x00, 0x41},	/* TVAR_ash_pGAS[255] */
	{0x0F, 0x12, 0x00, 0x53},	/* TVAR_ash_pGAS[256] */
	{0x0F, 0x12, 0x00, 0x6E},	/* TVAR_ash_pGAS[257] */
	{0x0F, 0x12, 0x00, 0x93},	/* TVAR_ash_pGAS[258] */
	{0x0F, 0x12, 0x00, 0xC7},	/* TVAR_ash_pGAS[259] */
	{0x0F, 0x12, 0x01, 0x33},	/* TVAR_ash_pGAS[260] */
	{0x0F, 0x12, 0x00, 0xE7},	/* TVAR_ash_pGAS[261] */
	{0x0F, 0x12, 0x00, 0xB0},	/* TVAR_ash_pGAS[262] */
	{0x0F, 0x12, 0x00, 0x90},	/* TVAR_ash_pGAS[263] */
	{0x0F, 0x12, 0x00, 0x78},	/* TVAR_ash_pGAS[264] */
	{0x0F, 0x12, 0x00, 0x69},	/* TVAR_ash_pGAS[265] */
	{0x0F, 0x12, 0x00, 0x60},	/* TVAR_ash_pGAS[266] */
	{0x0F, 0x12, 0x00, 0x61},	/* TVAR_ash_pGAS[267] */
	{0x0F, 0x12, 0x00, 0x6A},	/* TVAR_ash_pGAS[268] */
	{0x0F, 0x12, 0x00, 0x77},	/* TVAR_ash_pGAS[269] */
	{0x0F, 0x12, 0x00, 0x94},	/* TVAR_ash_pGAS[270] */
	{0x0F, 0x12, 0x00, 0xC3},	/* TVAR_ash_pGAS[271] */
	{0x0F, 0x12, 0x01, 0x01},	/* TVAR_ash_pGAS[272] */
	{0x0F, 0x12, 0x01, 0x74},	/* TVAR_ash_pGAS[273] */
	{0x0F, 0x12, 0x01, 0x11},	/* TVAR_ash_pGAS[274] */
	{0x0F, 0x12, 0x00, 0xCC},	/* TVAR_ash_pGAS[275] */
	{0x0F, 0x12, 0x00, 0xA3},	/* TVAR_ash_pGAS[276] */
	{0x0F, 0x12, 0x00, 0x8C},	/* TVAR_ash_pGAS[277] */
	{0x0F, 0x12, 0x00, 0x7E},	/* TVAR_ash_pGAS[278] */
	{0x0F, 0x12, 0x00, 0x76},	/* TVAR_ash_pGAS[279] */
	{0x0F, 0x12, 0x00, 0x7E},	/* TVAR_ash_pGAS[280] */
	{0x0F, 0x12, 0x00, 0x85},	/* TVAR_ash_pGAS[281] */
	{0x0F, 0x12, 0x00, 0x96},	/* TVAR_ash_pGAS[282] */
	{0x0F, 0x12, 0x00, 0xBB},	/* TVAR_ash_pGAS[283] */
	{0x0F, 0x12, 0x00, 0xF7},	/* TVAR_ash_pGAS[284] */
	{0x0F, 0x12, 0x01, 0x3E},	/* TVAR_ash_pGAS[285] */
	{0x0F, 0x12, 0x01, 0x4F},	/* TVAR_ash_pGAS[286] */
	{0x0F, 0x12, 0x00, 0xF9},	/* TVAR_ash_pGAS[287] */
	{0x0F, 0x12, 0x00, 0xC7},	/* TVAR_ash_pGAS[288] */
	{0x0F, 0x12, 0x00, 0xA2},	/* TVAR_ash_pGAS[289] */
	{0x0F, 0x12, 0x00, 0x8C},	/* TVAR_ash_pGAS[290] */
	{0x0F, 0x12, 0x00, 0x7E},	/* TVAR_ash_pGAS[291] */
	{0x0F, 0x12, 0x00, 0x78},	/* TVAR_ash_pGAS[292] */
	{0x0F, 0x12, 0x00, 0x79},	/* TVAR_ash_pGAS[293] */
	{0x0F, 0x12, 0x00, 0x83},	/* TVAR_ash_pGAS[294] */
	{0x0F, 0x12, 0x00, 0x93},	/* TVAR_ash_pGAS[295] */
	{0x0F, 0x12, 0x00, 0xB0},	/* TVAR_ash_pGAS[296] */
	{0x0F, 0x12, 0x00, 0xDC},	/* TVAR_ash_pGAS[297] */
	{0x0F, 0x12, 0x01, 0x13},	/* TVAR_ash_pGAS[298] */
	{0x0F, 0x12, 0x01, 0x12},	/* TVAR_ash_pGAS[299] */
	{0x0F, 0x12, 0x00, 0xD3},	/* TVAR_ash_pGAS[300] */
	{0x0F, 0x12, 0x00, 0xA2},	/* TVAR_ash_pGAS[301] */
	{0x0F, 0x12, 0x00, 0x86},	/* TVAR_ash_pGAS[302] */
	{0x0F, 0x12, 0x00, 0x6F},	/* TVAR_ash_pGAS[303] */
	{0x0F, 0x12, 0x00, 0x60},	/* TVAR_ash_pGAS[304] */
	{0x0F, 0x12, 0x00, 0x59},	/* TVAR_ash_pGAS[305] */
	{0x0F, 0x12, 0x00, 0x59},	/* TVAR_ash_pGAS[306] */
	{0x0F, 0x12, 0x00, 0x63},	/* TVAR_ash_pGAS[307] */
	{0x0F, 0x12, 0x00, 0x72},	/* TVAR_ash_pGAS[308] */
	{0x0F, 0x12, 0x00, 0x8C},	/* TVAR_ash_pGAS[309] */
	{0x0F, 0x12, 0x00, 0xB6},	/* TVAR_ash_pGAS[310] */
	{0x0F, 0x12, 0x00, 0xE8},	/* TVAR_ash_pGAS[311] */
	{0x0F, 0x12, 0x00, 0xED},	/* TVAR_ash_pGAS[312] */
	{0x0F, 0x12, 0x00, 0xB1},	/* TVAR_ash_pGAS[313] */
	{0x0F, 0x12, 0x00, 0x8B},	/* TVAR_ash_pGAS[314] */
	{0x0F, 0x12, 0x00, 0x6C},	/* TVAR_ash_pGAS[315] */
	{0x0F, 0x12, 0x00, 0x52},	/* TVAR_ash_pGAS[316] */
	{0x0F, 0x12, 0x00, 0x42},	/* TVAR_ash_pGAS[317] */
	{0x0F, 0x12, 0x00, 0x3D},	/* TVAR_ash_pGAS[318] */
	{0x0F, 0x12, 0x00, 0x38},	/* TVAR_ash_pGAS[319] */
	{0x0F, 0x12, 0x00, 0x3F},	/* TVAR_ash_pGAS[320] */
	{0x0F, 0x12, 0x00, 0x52},	/* TVAR_ash_pGAS[321] */
	{0x0F, 0x12, 0x00, 0x6A},	/* TVAR_ash_pGAS[322] */
	{0x0F, 0x12, 0x00, 0x90},	/* TVAR_ash_pGAS[323] */
	{0x0F, 0x12, 0x00, 0xBD},	/* TVAR_ash_pGAS[324] */
	{0x0F, 0x12, 0x00, 0xD2},	/* TVAR_ash_pGAS[325] */
	{0x0F, 0x12, 0x00, 0x9A},	/* TVAR_ash_pGAS[326] */
	{0x0F, 0x12, 0x00, 0x77},	/* TVAR_ash_pGAS[327] */
	{0x0F, 0x12, 0x00, 0x56},	/* TVAR_ash_pGAS[328] */
	{0x0F, 0x12, 0x00, 0x3C},	/* TVAR_ash_pGAS[329] */
	{0x0F, 0x12, 0x00, 0x24},	/* TVAR_ash_pGAS[330] */
	{0x0F, 0x12, 0x00, 0x1D},	/* TVAR_ash_pGAS[331] */
	{0x0F, 0x12, 0x00, 0x1E},	/* TVAR_ash_pGAS[332] */
	{0x0F, 0x12, 0x00, 0x25},	/* TVAR_ash_pGAS[333] */
	{0x0F, 0x12, 0x00, 0x3A},	/* TVAR_ash_pGAS[334] */
	{0x0F, 0x12, 0x00, 0x54},	/* TVAR_ash_pGAS[335] */
	{0x0F, 0x12, 0x00, 0x7A},	/* TVAR_ash_pGAS[336] */
	{0x0F, 0x12, 0x00, 0x9D},	/* TVAR_ash_pGAS[337] */
	{0x0F, 0x12, 0x00, 0xBF},	/* TVAR_ash_pGAS[338] */
	{0x0F, 0x12, 0x00, 0x8E},	/* TVAR_ash_pGAS[339] */
	{0x0F, 0x12, 0x00, 0x6E},	/* TVAR_ash_pGAS[340] */
	{0x0F, 0x12, 0x00, 0x49},	/* TVAR_ash_pGAS[341] */
	{0x0F, 0x12, 0x00, 0x2C},	/* TVAR_ash_pGAS[342] */
	{0x0F, 0x12, 0x00, 0x12},	/* TVAR_ash_pGAS[343] */
	{0x0F, 0x12, 0x00, 0x07},	/* TVAR_ash_pGAS[344] */
	{0x0F, 0x12, 0x00, 0x09},	/* TVAR_ash_pGAS[345] */
	{0x0F, 0x12, 0x00, 0x12},	/* TVAR_ash_pGAS[346] */
	{0x0F, 0x12, 0x00, 0x27},	/* TVAR_ash_pGAS[347] */
	{0x0F, 0x12, 0x00, 0x44},	/* TVAR_ash_pGAS[348] */
	{0x0F, 0x12, 0x00, 0x63},	/* TVAR_ash_pGAS[349] */
	{0x0F, 0x12, 0x00, 0x8F},	/* TVAR_ash_pGAS[350] */
	{0x0F, 0x12, 0x00, 0xBE},	/* TVAR_ash_pGAS[351] */
	{0x0F, 0x12, 0x00, 0x8C},	/* TVAR_ash_pGAS[352] */
	{0x0F, 0x12, 0x00, 0x69},	/* TVAR_ash_pGAS[353] */
	{0x0F, 0x12, 0x00, 0x46},	/* TVAR_ash_pGAS[354] */
	{0x0F, 0x12, 0x00, 0x25},	/* TVAR_ash_pGAS[355] */
	{0x0F, 0x12, 0x00, 0x0B},	/* TVAR_ash_pGAS[356] */
	{0x0F, 0x12, 0x00, 0x01},	/* TVAR_ash_pGAS[357] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_ash_pGAS[358] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_ash_pGAS[359] */
	{0x0F, 0x12, 0x00, 0x1F},	/* TVAR_ash_pGAS[360] */
	{0x0F, 0x12, 0x00, 0x3C},	/* TVAR_ash_pGAS[361] */
	{0x0F, 0x12, 0x00, 0x5E},	/* TVAR_ash_pGAS[362] */
	{0x0F, 0x12, 0x00, 0x88},	/* TVAR_ash_pGAS[363] */
	{0x0F, 0x12, 0x00, 0xC7},	/* TVAR_ash_pGAS[364] */
	{0x0F, 0x12, 0x00, 0x93},	/* TVAR_ash_pGAS[365] */
	{0x0F, 0x12, 0x00, 0x6F},	/* TVAR_ash_pGAS[366] */
	{0x0F, 0x12, 0x00, 0x4A},	/* TVAR_ash_pGAS[367] */
	{0x0F, 0x12, 0x00, 0x2B},	/* TVAR_ash_pGAS[368] */
	{0x0F, 0x12, 0x00, 0x13},	/* TVAR_ash_pGAS[369] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_ash_pGAS[370] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_ash_pGAS[371] */
	{0x0F, 0x12, 0x00, 0x0F},	/* TVAR_ash_pGAS[372] */
	{0x0F, 0x12, 0x00, 0x22},	/* TVAR_ash_pGAS[373] */
	{0x0F, 0x12, 0x00, 0x3D},	/* TVAR_ash_pGAS[374] */
	{0x0F, 0x12, 0x00, 0x61},	/* TVAR_ash_pGAS[375] */
	{0x0F, 0x12, 0x00, 0x89},	/* TVAR_ash_pGAS[376] */
	{0x0F, 0x12, 0x00, 0xDB},	/* TVAR_ash_pGAS[377] */
	{0x0F, 0x12, 0x00, 0xA3},	/* TVAR_ash_pGAS[378] */
	{0x0F, 0x12, 0x00, 0x7C},	/* TVAR_ash_pGAS[379] */
	{0x0F, 0x12, 0x00, 0x5A},	/* TVAR_ash_pGAS[380] */
	{0x0F, 0x12, 0x00, 0x3C},	/* TVAR_ash_pGAS[381] */
	{0x0F, 0x12, 0x00, 0x27},	/* TVAR_ash_pGAS[382] */
	{0x0F, 0x12, 0x00, 0x19},	/* TVAR_ash_pGAS[383] */
	{0x0F, 0x12, 0x00, 0x18},	/* TVAR_ash_pGAS[384] */
	{0x0F, 0x12, 0x00, 0x1F},	/* TVAR_ash_pGAS[385] */
	{0x0F, 0x12, 0x00, 0x31},	/* TVAR_ash_pGAS[386] */
	{0x0F, 0x12, 0x00, 0x49},	/* TVAR_ash_pGAS[387] */
	{0x0F, 0x12, 0x00, 0x6D},	/* TVAR_ash_pGAS[388] */
	{0x0F, 0x12, 0x00, 0x96},	/* TVAR_ash_pGAS[389] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_ash_pGAS[390] */
	{0x0F, 0x12, 0x00, 0xC3},	/* TVAR_ash_pGAS[391] */
	{0x0F, 0x12, 0x00, 0x94},	/* TVAR_ash_pGAS[392] */
	{0x0F, 0x12, 0x00, 0x74},	/* TVAR_ash_pGAS[393] */
	{0x0F, 0x12, 0x00, 0x57},	/* TVAR_ash_pGAS[394] */
	{0x0F, 0x12, 0x00, 0x43},	/* TVAR_ash_pGAS[395] */
	{0x0F, 0x12, 0x00, 0x37},	/* TVAR_ash_pGAS[396] */
	{0x0F, 0x12, 0x00, 0x35},	/* TVAR_ash_pGAS[397] */
	{0x0F, 0x12, 0x00, 0x3A},	/* TVAR_ash_pGAS[398] */
	{0x0F, 0x12, 0x00, 0x49},	/* TVAR_ash_pGAS[399] */
	{0x0F, 0x12, 0x00, 0x60},	/* TVAR_ash_pGAS[400] */
	{0x0F, 0x12, 0x00, 0x85},	/* TVAR_ash_pGAS[401] */
	{0x0F, 0x12, 0x00, 0xB5},	/* TVAR_ash_pGAS[402] */
	{0x0F, 0x12, 0x01, 0x41},	/* TVAR_ash_pGAS[403] */
	{0x0F, 0x12, 0x00, 0xF5},	/* TVAR_ash_pGAS[404] */
	{0x0F, 0x12, 0x00, 0xBA},	/* TVAR_ash_pGAS[405] */
	{0x0F, 0x12, 0x00, 0x99},	/* TVAR_ash_pGAS[406] */
	{0x0F, 0x12, 0x00, 0x7D},	/* TVAR_ash_pGAS[407] */
	{0x0F, 0x12, 0x00, 0x6A},	/* TVAR_ash_pGAS[408] */
	{0x0F, 0x12, 0x00, 0x5E},	/* TVAR_ash_pGAS[409] */
	{0x0F, 0x12, 0x00, 0x5C},	/* TVAR_ash_pGAS[410] */
	{0x0F, 0x12, 0x00, 0x61},	/* TVAR_ash_pGAS[411] */
	{0x0F, 0x12, 0x00, 0x6B},	/* TVAR_ash_pGAS[412] */
	{0x0F, 0x12, 0x00, 0x85},	/* TVAR_ash_pGAS[413] */
	{0x0F, 0x12, 0x00, 0xB1},	/* TVAR_ash_pGAS[414] */
	{0x0F, 0x12, 0x00, 0xEB},	/* TVAR_ash_pGAS[415] */
	{0x0F, 0x12, 0x01, 0x82},	/* TVAR_ash_pGAS[416] */
	{0x0F, 0x12, 0x01, 0x1E},	/* TVAR_ash_pGAS[417] */
	{0x0F, 0x12, 0x00, 0xD7},	/* TVAR_ash_pGAS[418] */
	{0x0F, 0x12, 0x00, 0xAC},	/* TVAR_ash_pGAS[419] */
	{0x0F, 0x12, 0x00, 0x91},	/* TVAR_ash_pGAS[420] */
	{0x0F, 0x12, 0x00, 0x80},	/* TVAR_ash_pGAS[421] */
	{0x0F, 0x12, 0x00, 0x76},	/* TVAR_ash_pGAS[422] */
	{0x0F, 0x12, 0x00, 0x79},	/* TVAR_ash_pGAS[423] */
	{0x0F, 0x12, 0x00, 0x7D},	/* TVAR_ash_pGAS[424] */
	{0x0F, 0x12, 0x00, 0x89},	/* TVAR_ash_pGAS[425] */
	{0x0F, 0x12, 0x00, 0xAB},	/* TVAR_ash_pGAS[426] */
	{0x0F, 0x12, 0x00, 0xE6},	/* TVAR_ash_pGAS[427] */
	{0x0F, 0x12, 0x01, 0x24},	/* TVAR_ash_pGAS[428] */
	{0x0F, 0x12, 0x00, 0xFB},	/* TVAR_ash_pGAS[429] */
	{0x0F, 0x12, 0x00, 0xBB},	/* TVAR_ash_pGAS[430] */
	{0x0F, 0x12, 0x00, 0x93},	/* TVAR_ash_pGAS[431] */
	{0x0F, 0x12, 0x00, 0x79},	/* TVAR_ash_pGAS[432] */
	{0x0F, 0x12, 0x00, 0x6A},	/* TVAR_ash_pGAS[433] */
	{0x0F, 0x12, 0x00, 0x61},	/* TVAR_ash_pGAS[434] */
	{0x0F, 0x12, 0x00, 0x5E},	/* TVAR_ash_pGAS[435] */
	{0x0F, 0x12, 0x00, 0x60},	/* TVAR_ash_pGAS[436] */
	{0x0F, 0x12, 0x00, 0x67},	/* TVAR_ash_pGAS[437] */
	{0x0F, 0x12, 0x00, 0x72},	/* TVAR_ash_pGAS[438] */
	{0x0F, 0x12, 0x00, 0x8A},	/* TVAR_ash_pGAS[439] */
	{0x0F, 0x12, 0x00, 0xAD},	/* TVAR_ash_pGAS[440] */
	{0x0F, 0x12, 0x00, 0xDB},	/* TVAR_ash_pGAS[441] */
	{0x0F, 0x12, 0x00, 0xCD},	/* TVAR_ash_pGAS[442] */
	{0x0F, 0x12, 0x00, 0x9C},	/* TVAR_ash_pGAS[443] */
	{0x0F, 0x12, 0x00, 0x76},	/* TVAR_ash_pGAS[444] */
	{0x0F, 0x12, 0x00, 0x64},	/* TVAR_ash_pGAS[445] */
	{0x0F, 0x12, 0x00, 0x53},	/* TVAR_ash_pGAS[446] */
	{0x0F, 0x12, 0x00, 0x4A},	/* TVAR_ash_pGAS[447] */
	{0x0F, 0x12, 0x00, 0x45},	/* TVAR_ash_pGAS[448] */
	{0x0F, 0x12, 0x00, 0x46},	/* TVAR_ash_pGAS[449] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_ash_pGAS[450] */
	{0x0F, 0x12, 0x00, 0x59},	/* TVAR_ash_pGAS[451] */
	{0x0F, 0x12, 0x00, 0x6D},	/* TVAR_ash_pGAS[452] */
	{0x0F, 0x12, 0x00, 0x91},	/* TVAR_ash_pGAS[453] */
	{0x0F, 0x12, 0x00, 0xBB},	/* TVAR_ash_pGAS[454] */
	{0x0F, 0x12, 0x00, 0xA9},	/* TVAR_ash_pGAS[455] */
	{0x0F, 0x12, 0x00, 0x7D},	/* TVAR_ash_pGAS[456] */
	{0x0F, 0x12, 0x00, 0x62},	/* TVAR_ash_pGAS[457] */
	{0x0F, 0x12, 0x00, 0x4D},	/* TVAR_ash_pGAS[458] */
	{0x0F, 0x12, 0x00, 0x3C},	/* TVAR_ash_pGAS[459] */
	{0x0F, 0x12, 0x00, 0x31},	/* TVAR_ash_pGAS[460] */
	{0x0F, 0x12, 0x00, 0x2F},	/* TVAR_ash_pGAS[461] */
	{0x0F, 0x12, 0x00, 0x2C},	/* TVAR_ash_pGAS[462] */
	{0x0F, 0x12, 0x00, 0x31},	/* TVAR_ash_pGAS[463] */
	{0x0F, 0x12, 0x00, 0x40},	/* TVAR_ash_pGAS[464] */
	{0x0F, 0x12, 0x00, 0x52},	/* TVAR_ash_pGAS[465] */
	{0x0F, 0x12, 0x00, 0x6F},	/* TVAR_ash_pGAS[466] */
	{0x0F, 0x12, 0x00, 0x97},	/* TVAR_ash_pGAS[467] */
	{0x0F, 0x12, 0x00, 0x95},	/* TVAR_ash_pGAS[468] */
	{0x0F, 0x12, 0x00, 0x6A},	/* TVAR_ash_pGAS[469] */
	{0x0F, 0x12, 0x00, 0x52},	/* TVAR_ash_pGAS[470] */
	{0x0F, 0x12, 0x00, 0x3C},	/* TVAR_ash_pGAS[471] */
	{0x0F, 0x12, 0x00, 0x2B},	/* TVAR_ash_pGAS[472] */
	{0x0F, 0x12, 0x00, 0x1A},	/* TVAR_ash_pGAS[473] */
	{0x0F, 0x12, 0x00, 0x16},	/* TVAR_ash_pGAS[474] */
	{0x0F, 0x12, 0x00, 0x18},	/* TVAR_ash_pGAS[475] */
	{0x0F, 0x12, 0x00, 0x1E},	/* TVAR_ash_pGAS[476] */
	{0x0F, 0x12, 0x00, 0x2B},	/* TVAR_ash_pGAS[477] */
	{0x0F, 0x12, 0x00, 0x3F},	/* TVAR_ash_pGAS[478] */
	{0x0F, 0x12, 0x00, 0x5C},	/* TVAR_ash_pGAS[479] */
	{0x0F, 0x12, 0x00, 0x78},	/* TVAR_ash_pGAS[480] */
	{0x0F, 0x12, 0x00, 0x88},	/* TVAR_ash_pGAS[481] */
	{0x0F, 0x12, 0x00, 0x62},	/* TVAR_ash_pGAS[482] */
	{0x0F, 0x12, 0x00, 0x4C},	/* TVAR_ash_pGAS[483] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_ash_pGAS[484] */
	{0x0F, 0x12, 0x00, 0x1E},	/* TVAR_ash_pGAS[485] */
	{0x0F, 0x12, 0x00, 0x0B},	/* TVAR_ash_pGAS[486] */
	{0x0F, 0x12, 0x00, 0x04},	/* TVAR_ash_pGAS[487] */
	{0x0F, 0x12, 0x00, 0x07},	/* TVAR_ash_pGAS[488] */
	{0x0F, 0x12, 0x00, 0x0C},	/* TVAR_ash_pGAS[489] */
	{0x0F, 0x12, 0x00, 0x1C},	/* TVAR_ash_pGAS[490] */
	{0x0F, 0x12, 0x00, 0x31},	/* TVAR_ash_pGAS[491] */
	{0x0F, 0x12, 0x00, 0x47},	/* TVAR_ash_pGAS[492] */
	{0x0F, 0x12, 0x00, 0x6C},	/* TVAR_ash_pGAS[493] */
	{0x0F, 0x12, 0x00, 0x85},	/* TVAR_ash_pGAS[494] */
	{0x0F, 0x12, 0x00, 0x60},	/* TVAR_ash_pGAS[495] */
	{0x0F, 0x12, 0x00, 0x4A},	/* TVAR_ash_pGAS[496] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_ash_pGAS[497] */
	{0x0F, 0x12, 0x00, 0x19},	/* TVAR_ash_pGAS[498] */
	{0x0F, 0x12, 0x00, 0x07},	/* TVAR_ash_pGAS[499] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_ash_pGAS[500] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_ash_pGAS[501] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_ash_pGAS[502] */
	{0x0F, 0x12, 0x00, 0x14},	/* TVAR_ash_pGAS[503] */
	{0x0F, 0x12, 0x00, 0x29},	/* TVAR_ash_pGAS[504] */
	{0x0F, 0x12, 0x00, 0x42},	/* TVAR_ash_pGAS[505] */
	{0x0F, 0x12, 0x00, 0x65},	/* TVAR_ash_pGAS[506] */
	{0x0F, 0x12, 0x00, 0x8F},	/* TVAR_ash_pGAS[507] */
	{0x0F, 0x12, 0x00, 0x67},	/* TVAR_ash_pGAS[508] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_ash_pGAS[509] */
	{0x0F, 0x12, 0x00, 0x36},	/* TVAR_ash_pGAS[510] */
	{0x0F, 0x12, 0x00, 0x1F},	/* TVAR_ash_pGAS[511] */
	{0x0F, 0x12, 0x00, 0x0F},	/* TVAR_ash_pGAS[512] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_ash_pGAS[513] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_ash_pGAS[514] */
	{0x0F, 0x12, 0x00, 0x0B},	/* TVAR_ash_pGAS[515] */
	{0x0F, 0x12, 0x00, 0x17},	/* TVAR_ash_pGAS[516] */
	{0x0F, 0x12, 0x00, 0x2A},	/* TVAR_ash_pGAS[517] */
	{0x0F, 0x12, 0x00, 0x44},	/* TVAR_ash_pGAS[518] */
	{0x0F, 0x12, 0x00, 0x65},	/* TVAR_ash_pGAS[519] */
	{0x0F, 0x12, 0x00, 0x9F},	/* TVAR_ash_pGAS[520] */
	{0x0F, 0x12, 0x00, 0x76},	/* TVAR_ash_pGAS[521] */
	{0x0F, 0x12, 0x00, 0x5C},	/* TVAR_ash_pGAS[522] */
	{0x0F, 0x12, 0x00, 0x44},	/* TVAR_ash_pGAS[523] */
	{0x0F, 0x12, 0x00, 0x2F},	/* TVAR_ash_pGAS[524] */
	{0x0F, 0x12, 0x00, 0x22},	/* TVAR_ash_pGAS[525] */
	{0x0F, 0x12, 0x00, 0x15},	/* TVAR_ash_pGAS[526] */
	{0x0F, 0x12, 0x00, 0x15},	/* TVAR_ash_pGAS[527] */
	{0x0F, 0x12, 0x00, 0x17},	/* TVAR_ash_pGAS[528] */
	{0x0F, 0x12, 0x00, 0x24},	/* TVAR_ash_pGAS[529] */
	{0x0F, 0x12, 0x00, 0x35},	/* TVAR_ash_pGAS[530] */
	{0x0F, 0x12, 0x00, 0x4F},	/* TVAR_ash_pGAS[531] */
	{0x0F, 0x12, 0x00, 0x70},	/* TVAR_ash_pGAS[532] */
	{0x0F, 0x12, 0x0, 0x0C1},	/* TVAR_ash_pGAS[533] */
	{0x0F, 0x12, 0x00, 0x93},	/* TVAR_ash_pGAS[534] */
	{0x0F, 0x12, 0x00, 0x71},	/* TVAR_ash_pGAS[535] */
	{0x0F, 0x12, 0x00, 0x5C},	/* TVAR_ash_pGAS[536] */
	{0x0F, 0x12, 0x00, 0x47},	/* TVAR_ash_pGAS[537] */
	{0x0F, 0x12, 0x00, 0x3A},	/* TVAR_ash_pGAS[538] */
	{0x0F, 0x12, 0x00, 0x30},	/* TVAR_ash_pGAS[539] */
	{0x0F, 0x12, 0x00, 0x2E},	/* TVAR_ash_pGAS[540] */
	{0x0F, 0x12, 0x00, 0x30},	/* TVAR_ash_pGAS[541] */
	{0x0F, 0x12, 0x00, 0x38},	/* TVAR_ash_pGAS[542] */
	{0x0F, 0x12, 0x00, 0x4A},	/* TVAR_ash_pGAS[543] */
	{0x0F, 0x12, 0x00, 0x64},	/* TVAR_ash_pGAS[544] */
	{0x0F, 0x12, 0x0, 0x08D},	/* TVAR_ash_pGAS[545] */
	{0x0F, 0x12, 0x00, 0xFF},	/* TVAR_ash_pGAS[546] */
	{0x0F, 0x12, 0x00, 0xC0},	/* TVAR_ash_pGAS[547] */
	{0x0F, 0x12, 0x00, 0x94},	/* TVAR_ash_pGAS[548] */
	{0x0F, 0x12, 0x00, 0x7B},	/* TVAR_ash_pGAS[549] */
	{0x0F, 0x12, 0x00, 0x6A},	/* TVAR_ash_pGAS[550] */
	{0x0F, 0x12, 0x00, 0x5D},	/* TVAR_ash_pGAS[551] */
	{0x0F, 0x12, 0x00, 0x53},	/* TVAR_ash_pGAS[552] */
	{0x0F, 0x12, 0x00, 0x52},	/* TVAR_ash_pGAS[553] */
	{0x0F, 0x12, 0x0, 0x053},	/* TVAR_ash_pGAS[554] */
	{0x0F, 0x12, 0x0, 0x057},	/* TVAR_ash_pGAS[555] */
	{0x0F, 0x12, 0x00, 0x6A},	/* TVAR_ash_pGAS[556] */
	{0x0F, 0x12, 0x00, 0x8E},	/* TVAR_ash_pGAS[557] */
	{0x0F, 0x12, 0x00, 0xC0},	/* TVAR_ash_pGAS[558] */
	{0x0F, 0x12, 0x01, 0x2E},	/* TVAR_ash_pGAS[559] */
	{0x0F, 0x12, 0x00, 0xDC},	/* TVAR_ash_pGAS[560] */
	{0x0F, 0x12, 0x00, 0xA6},	/* TVAR_ash_pGAS[561] */
	{0x0F, 0x12, 0x00, 0x86},	/* TVAR_ash_pGAS[562] */
	{0x0F, 0x12, 0x00, 0x73},	/* TVAR_ash_pGAS[563] */
	{0x0F, 0x12, 0x00, 0x6A},	/* TVAR_ash_pGAS[564] */
	{0x0F, 0x12, 0x00, 0x62},	/* TVAR_ash_pGAS[565] */
	{0x0F, 0x12, 0x00, 0x64},	/* TVAR_ash_pGAS[566] */
	{0x0F, 0x12, 0x00, 0x65},	/* TVAR_ash_pGAS[567] */
	{0x0F, 0x12, 0x00, 0x6C},	/* TVAR_ash_pGAS[568] */
	{0x0F, 0x12, 0x00, 0x88},	/* TVAR_ash_pGAS[569] */
	{0x0F, 0x12, 0x00, 0xBA},	/* TVAR_ash_pGAS[570] */
	{0x0F, 0x12, 0x00, 0xEF},	/* TVAR_ash_pGAS[571] */

	{0x00, 0x2A, 0x0F, 0x12},
	{0x0F, 0x12, 0x02, 0xC9},	/* awbb_GLocusR */
	{0x0F, 0x12, 0x03, 0x3F},	/* awbb_GLocusB */

	/* param_start	TVAR_ash_AwbAshCord */
	{0x00, 0x2A, 0x07, 0x04},
	{0x0F, 0x12, 0x00, 0xC7},	/* TVAR_ash_AwbAshCord[0] */
	{0x0F, 0x12, 0x00, 0xF7},	/* TVAR_ash_AwbAshCord[1] */
	{0x0F, 0x12, 0x01, 0x07},	/* TVAR_ash_AwbAshCord[2] */
	{0x0F, 0x12, 0x01, 0x42},	/* TVAR_ash_AwbAshCord[3] */
	{0x0F, 0x12, 0x01, 0x7A},	/* TVAR_ash_AwbAshCord[4] */
	{0x0F, 0x12, 0x01, 0xA0},	/* TVAR_ash_AwbAshCord[5] */
	{0x0F, 0x12, 0x01, 0xB6},	/* TVAR_ash_AwbAshCord[6] */
	/* param_end	TVAR_ash_AwbAshCord */

	{0x00, 0x2A, 0x07, 0x54},
	{0x0F, 0x12, 0x24, 0x7C},
	{0x0F, 0x12, 0x70, 0x00},

	/* param_start	awbb_IndoorGrZones_m_BGrid */
	{0x00, 0x2A, 0x0C, 0x50},
	{0x0F, 0x12, 0x03, 0xBA},
	{0x0F, 0x12, 0x03, 0xE3},
	{0x0F, 0x12, 0x03, 0x9A},
	{0x0F, 0x12, 0x03, 0xDB},
	{0x0F, 0x12, 0x03, 0x7B},
	{0x0F, 0x12, 0x03, 0xCD},
	{0x0F, 0x12, 0x03, 0x5D},
	{0x0F, 0x12, 0x03, 0xB2},
	{0x0F, 0x12, 0x03, 0x42},
	{0x0F, 0x12, 0x03, 0x97},
	{0x0F, 0x12, 0x03, 0x25},
	{0x0F, 0x12, 0x03, 0x80},
	{0x0F, 0x12, 0x03, 0x0E},
	{0x0F, 0x12, 0x03, 0x69},
	{0x0F, 0x12, 0x02, 0xF8},
	{0x0F, 0x12, 0x03, 0x4B},
	{0x0F, 0x12, 0x02, 0xDE},
	{0x0F, 0x12, 0x03, 0x36},
	{0x0F, 0x12, 0x02, 0xBF},
	{0x0F, 0x12, 0x03, 0x1F},
	{0x0F, 0x12, 0x02, 0xA6},
	{0x0F, 0x12, 0x03, 0x06},
	{0x0F, 0x12, 0x02, 0x8D},
	{0x0F, 0x12, 0x02, 0xF4},
	{0x0F, 0x12, 0x02, 0x7D},
	{0x0F, 0x12, 0x02, 0xDD},
	{0x0F, 0x12, 0x02, 0x6C},
	{0x0F, 0x12, 0x02, 0xC2},
	{0x0F, 0x12, 0x02, 0x5C},
	{0x0F, 0x12, 0x02, 0xAE},
	{0x0F, 0x12, 0x02, 0x4F},
	{0x0F, 0x12, 0x02, 0x9D},
	{0x0F, 0x12, 0x02, 0x45},
	{0x0F, 0x12, 0x02, 0x8B},
	{0x0F, 0x12, 0x02, 0x3E},
	{0x0F, 0x12, 0x02, 0x7F},
	{0x0F, 0x12, 0x02, 0x35},
	{0x0F, 0x12, 0x02, 0x72},
	{0x0F, 0x12, 0x02, 0x2B},
	{0x0F, 0x12, 0x02, 0x67},
	{0x0F, 0x12, 0x02, 0x20},
	{0x0F, 0x12, 0x02, 0x5B},
	{0x0F, 0x12, 0x02, 0x18},
	{0x0F, 0x12, 0x02, 0x50},
	{0x0F, 0x12, 0x02, 0x0E},
	{0x0F, 0x12, 0x02, 0x46},
	{0x0F, 0x12, 0x02, 0x06},
	{0x0F, 0x12, 0x02, 0x3D},
	{0x0F, 0x12, 0x01, 0xFB},
	{0x0F, 0x12, 0x02, 0x34},
	{0x0F, 0x12, 0x01, 0xF1},
	{0x0F, 0x12, 0x02, 0x29},
	{0x0F, 0x12, 0x01, 0xE7},
	{0x0F, 0x12, 0x02, 0x20},
	{0x0F, 0x12, 0x01, 0xDF},
	{0x0F, 0x12, 0x02, 0x16},
	{0x0F, 0x12, 0x01, 0xDC},
	{0x0F, 0x12, 0x02, 0x0D},
	{0x0F, 0x12, 0x01, 0xE1},
	{0x0F, 0x12, 0x01, 0xFD},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	/*	param_end	awbb_IndoorGrZones_m_BGrid */
	{0x0F, 0x12, 0x00, 0x04},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x0C, 0xF8},
	{0x0F, 0x12, 0x01, 0x0F},
	{0x0F, 0x12, 0x00, 0x00},
	/*	param_start	awbb_LowBrGrZones_m_BGrid */
	{0x00, 0x2A, 0x0D, 0x84},
	{0x0F, 0x12, 0x04, 0x06},
	{0x0F, 0x12, 0x04, 0x67},
	{0x0F, 0x12, 0x03, 0x71},
	{0x0F, 0x12, 0x04, 0xB0},
	{0x0F, 0x12, 0x02, 0xE5},
	{0x0F, 0x12, 0x04, 0x81},
	{0x0F, 0x12, 0x02, 0x98},
	{0x0F, 0x12, 0x04, 0x2E},
	{0x0F, 0x12, 0x02, 0x60},
	{0x0F, 0x12, 0x03, 0xDE},
	{0x0F, 0x12, 0x02, 0x2F},
	{0x0F, 0x12, 0x03, 0x91},
	{0x0F, 0x12, 0x02, 0x01},
	{0x0F, 0x12, 0x03, 0x4D},
	{0x0F, 0x12, 0x01, 0xDA},
	{0x0F, 0x12, 0x03, 0x10},
	{0x0F, 0x12, 0x01, 0xB3},
	{0x0F, 0x12, 0x02, 0xD4},
	{0x0F, 0x12, 0x01, 0x8F},
	{0x0F, 0x12, 0x02, 0x97},
	{0x0F, 0x12, 0x01, 0x81},
	{0x0F, 0x12, 0x02, 0x71},
	{0x0F, 0x12, 0x01, 0x81},
	{0x0F, 0x12, 0x02, 0x2A},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	/* param_end	awbb_LowBrGrZones_m_BGrid */
	{0x0F, 0x12, 0x00, 0x06},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x0D, 0xF0},
	{0x0F, 0x12, 0x00, 0x81},
	{0x0F, 0x12, 0x00, 0x00},
	/* param_start	awbb_OutdoorGrZones_m_BGrid */
	{0x00, 0x2A, 0x0D, 0x08},
	{0x0F, 0x12, 0x02, 0x59},
	{0x0F, 0x12, 0x02, 0x68},
	{0x0F, 0x12, 0x02, 0x4F},
	{0x0F, 0x12, 0x02, 0x6C},
	{0x0F, 0x12, 0x02, 0x46},
	{0x0F, 0x12, 0x02, 0x6B},
	{0x0F, 0x12, 0x02, 0x40},
	{0x0F, 0x12, 0x02, 0x66},
	{0x0F, 0x12, 0x02, 0x39},
	{0x0F, 0x12, 0x02, 0x5E},
	{0x0F, 0x12, 0x02, 0x32},
	{0x0F, 0x12, 0x02, 0x58},
	{0x0F, 0x12, 0x02, 0x2D},
	{0x0F, 0x12, 0x02, 0x53},
	{0x0F, 0x12, 0x02, 0x27},
	{0x0F, 0x12, 0x02, 0x4D},
	{0x0F, 0x12, 0x02, 0x23},
	{0x0F, 0x12, 0x02, 0x47},
	{0x0F, 0x12, 0x02, 0x1F},
	{0x0F, 0x12, 0x02, 0x41},
	{0x0F, 0x12, 0x02, 0x19},
	{0x0F, 0x12, 0x02, 0x3C},
	{0x0F, 0x12, 0x02, 0x15},
	{0x0F, 0x12, 0x02, 0x36},
	{0x0F, 0x12, 0x02, 0x10},
	{0x0F, 0x12, 0x02, 0x31},
	{0x0F, 0x12, 0x02, 0x0B},
	{0x0F, 0x12, 0x02, 0x2D},
	{0x0F, 0x12, 0x02, 0x08},
	{0x0F, 0x12, 0x02, 0x27},
	{0x0F, 0x12, 0x02, 0x05},
	{0x0F, 0x12, 0x02, 0x20},
	{0x0F, 0x12, 0x02, 0x06},
	{0x0F, 0x12, 0x02, 0x16},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	/* param_end	awbb_OutdoorGrZones_m_BGrid */
	{0x0F, 0x12, 0x00, 0x03},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x0D, 0x74},
	{0x0F, 0x12, 0x02, 0x4C},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x0E, 0x00},
	{0x0F, 0x12, 0x03, 0x4A},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x01, 0x76},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x71, 0xB8},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x0E, 0x1A},
	{0x0F, 0x12, 0x01, 0x2F},
	{0x0F, 0x12, 0x01, 0x20},

	/* awbb_LowTempRB */
	{0x00, 0x2A, 0x0E, 0x68},
	{0x0F, 0x12, 0x04, 0xF2},


	{0x00, 0x2A, 0x0E, 0x42},
	{0x0F, 0x12, 0x00, 0x02},

	{0x00, 0x2A, 0x0E, 0xE0},
	{0x0F, 0x12, 0x00, 0xB5},
	{0x0F, 0x12, 0x00, 0xB5},
	{0x00, 0x2A, 0x0E, 0xD0},
	{0x0F, 0x12, 0x0F, 0xA6},
	{0x0F, 0x12, 0x10, 0x00},
	{0x0F, 0x12, 0x10, 0x64},
	{0x0F, 0x12, 0x11, 0x1F},
	{0x0F, 0x12, 0x11, 0x51},
	{0x0F, 0x12, 0x11, 0xBF},
	{0x0F, 0x12, 0x00, 0xB8},
	{0x0F, 0x12, 0x00, 0xB2},
	{0x00, 0x2A, 0x0E, 0xCA},
	{0x0F, 0x12, 0x03, 0x08},
	{0x0F, 0x12, 0x03, 0x44},
	{0x0F, 0x12, 0x03, 0x80},


	{0x00, 0x2A, 0x0E, 0x82},
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[0][0] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[0][1] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[0][2] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[0][3] */
	{0x0F, 0x12, 0x00, 0x60},	/* awbb_GridCorr_R[0][4] */
	{0x0F, 0x12, 0x01, 0x70},	/* awbb_GridCorr_R[0][5] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[1][0] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[1][1] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[1][2] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[1][3] */
	{0x0F, 0x12, 0x00, 0x60},	/* awbb_GridCorr_R[1][4] */
	{0x0F, 0x12, 0x01, 0x70},	/* awbb_GridCorr_R[1][5] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[2][0] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[2][1] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[2][2] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_R[2][3] */
	{0x0F, 0x12, 0x00, 0x60},	/* awbb_GridCorr_R[2][4] */
	{0x0F, 0x12, 0x01, 0x70},	/* awbb_GridCorr_R[2][5] */

	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[0][0] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[0][1] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[0][2] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[0][3] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[0][4] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[0][5] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[1][0] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[1][1] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[1][2] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[1][3] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[1][4] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[1][5] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[2][0] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[2][1] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[2][2] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[2][3] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[2][4] */
	{0x0F, 0x12, 0x00, 0x00},	/* awbb_GridCorr_B[2][5] */



	/* param_start	SARR_usGammaLutRGBIndoor */
	{0x00, 0x2A, 0x04, 0xC8},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x02},
	{0x0F, 0x12, 0x00, 0x04},
	{0x0F, 0x12, 0x00, 0x09},
	{0x0F, 0x12, 0x00, 0x18},
	{0x0F, 0x12, 0x00, 0xB8},
	{0x0F, 0x12, 0x01, 0x39},
	{0x0F, 0x12, 0x01, 0x88},
	{0x0F, 0x12, 0x01, 0xDF},
	{0x0F, 0x12, 0x02, 0x2D},
	{0x0F, 0x12, 0x02, 0xB5},
	{0x0F, 0x12, 0x03, 0x26},
	{0x0F, 0x12, 0x03, 0x76},
	{0x0F, 0x12, 0x03, 0xAC},
	{0x0F, 0x12, 0x03, 0xCA},
	{0x0F, 0x12, 0x03, 0xDE},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x04},
	{0x0F, 0x12, 0x00, 0x08},
	{0x0F, 0x12, 0x00, 0x10},
	{0x0F, 0x12, 0x00, 0x2D},
	{0x0F, 0x12, 0x00, 0xC8},
	{0x0F, 0x12, 0x01, 0x39},
	{0x0F, 0x12, 0x01, 0x88},
	{0x0F, 0x12, 0x01, 0xDF},
	{0x0F, 0x12, 0x02, 0x2D},
	{0x0F, 0x12, 0x02, 0xB5},
	{0x0F, 0x12, 0x03, 0x26},
	{0x0F, 0x12, 0x03, 0x76},
	{0x0F, 0x12, 0x03, 0xAC},
	{0x0F, 0x12, 0x03, 0xCA},
	{0x0F, 0x12, 0x03, 0xDE},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x04},
	{0x0F, 0x12, 0x00, 0x08},
	{0x0F, 0x12, 0x00, 0x10},
	{0x0F, 0x12, 0x00, 0x2D},
	{0x0F, 0x12, 0x00, 0xC8},
	{0x0F, 0x12, 0x01, 0x39},
	{0x0F, 0x12, 0x01, 0x88},
	{0x0F, 0x12, 0x01, 0xDF},
	{0x0F, 0x12, 0x02, 0x2D},
	{0x0F, 0x12, 0x02, 0xB5},
	{0x0F, 0x12, 0x03, 0x26},
	{0x0F, 0x12, 0x03, 0x76},
	{0x0F, 0x12, 0x03, 0xAC},
	{0x0F, 0x12, 0x03, 0xCA},
	{0x0F, 0x12, 0x03, 0xDE},
	/* param_end	SARR_usGammaLutRGBIndoor */


	{0x00, 0x2A, 0x10, 0x00},
	{0x0F, 0x12, 0x00, 0x3E},	/* 0036 */

	{0x00, 0x2A, 0x10, 0x8E},
	{0x0F, 0x12, 0x00, 0xC7},
	{0x0F, 0x12, 0x00, 0xF7},
	{0x0F, 0x12, 0x01, 0x07},
	{0x0F, 0x12, 0x01, 0x42},
	{0x0F, 0x12, 0x01, 0x7A},
	{0x0F, 0x12, 0x01, 0xA0},
	{0x0F, 0x12, 0x01, 0xB6},

	{0x0F, 0x12, 0x01, 0x00},	/* 0112 */
	{0x0F, 0x12, 0x01, 0x00},	/* 0122 */
	{0x0F, 0x12, 0x01, 0x00},	/* 0136 */
	{0x0F, 0x12, 0x01, 0x00},	/* 00F6 */
	{0x0F, 0x12, 0x01, 0x00},	/* 0100 */
	{0x0F, 0x12, 0x01, 0x00},	/* 00FE */
	{0x0F, 0x12, 0x01, 0x00},	/* 0100 */

	/*	param_start	TVAR_wbt_pBaseCcms */
	{0x00, 0x2A, 0x23, 0xA4},
	{0x0F, 0x12, 0x02, 0x04},	/*01AF */
	{0x0F, 0x12, 0xFF, 0xA2},	/*FFD5 */
	{0x0F, 0x12, 0xFF, 0xF3},	/*001D */
	{0x0F, 0x12, 0x00, 0xDA},	/*005F */
	{0x0F, 0x12, 0x01, 0x54},	/*00F6 */
	{0x0F, 0x12, 0xFF, 0x6D},	/*FF46 */
	{0x0F, 0x12, 0xFE, 0xF9},	/*FF2D */
	{0x0F, 0x12, 0x02, 0x09},	/*016B */
	{0x0F, 0x12, 0xFF, 0x3C},	/*FF83 */
	{0x0F, 0x12, 0xFF, 0x2C},	/*FF0F */
	{0x0F, 0x12, 0x00, 0xE7},	/*019A */
	{0x0F, 0x12, 0x00, 0xA5},	/*0117 */
	{0x0F, 0x12, 0xFF, 0xBE},	/*0005 */
	{0x0F, 0x12, 0xFF, 0xDE},	/*0020 */
	{0x0F, 0x12, 0x02, 0x0A},	/*01CC */
	{0x0F, 0x12, 0x00, 0xD4},	/*0140 */
	{0x0F, 0x12, 0xFF, 0x55},	/*FF46 */
	{0x0F, 0x12, 0x01, 0xE8},	/*0136 */

	{0x0F, 0x12, 0x02, 0x04},	/*01C5 */
	{0x0F, 0x12, 0xFF, 0xA2},	/*FFB8 */
	{0x0F, 0x12, 0xFF, 0xF3},	/*FFEA */
	{0x0F, 0x12, 0x00, 0xDA},	/*0199 */
	{0x0F, 0x12, 0x01, 0x54},	/*00DE */
	{0x0F, 0x12, 0xFF, 0x6D},	/*FEDE */
	{0x0F, 0x12, 0xFE, 0xF9},	/*FF58 */
	{0x0F, 0x12, 0x02, 0x09},	/*0214 */
	{0x0F, 0x12, 0xFF, 0x3C},	/*FF2C */
	{0x0F, 0x12, 0xFF, 0x2C},	/*FEBC */
	{0x0F, 0x12, 0x00, 0xE7},	/*01CC */
	{0x0F, 0x12, 0x00, 0xA5},	/*00BC */
	{0x0F, 0x12, 0xFF, 0xBE},	/*FFE4 */
	{0x0F, 0x12, 0xFF, 0xDE},	/*0004 */
	{0x0F, 0x12, 0x02, 0x0A},	/*01FF */
	{0x0F, 0x12, 0x00, 0xD4},	/*016E */
	{0x0F, 0x12, 0xFF, 0x55},	/*FF56 */
	{0x0F, 0x12, 0x01, 0xE8},	/*01BD */

	{0x0F, 0x12, 0x02, 0x04},	/*01C5 */
	{0x0F, 0x12, 0xFF, 0xA2},	/*FFB8 */
	{0x0F, 0x12, 0xFF, 0xF3},	/*FFEA */
	{0x0F, 0x12, 0x00, 0xDA},	/*0199 */
	{0x0F, 0x12, 0x01, 0x54},	/*00DE */
	{0x0F, 0x12, 0xFF, 0x6D},	/*FEDE */
	{0x0F, 0x12, 0xFE, 0xF9},	/*FF58 */
	{0x0F, 0x12, 0x02, 0x09},	/*0214 */
	{0x0F, 0x12, 0xFF, 0x3C},	/*FF2C */
	{0x0F, 0x12, 0xFF, 0x2C},	/*FEBC */
	{0x0F, 0x12, 0x00, 0xE7},	/*01CC */
	{0x0F, 0x12, 0x00, 0xA5},	/*00BC */
	{0x0F, 0x12, 0xFF, 0xBE},	/*FFE4 */
	{0x0F, 0x12, 0xFF, 0xDE},	/*0004 */
	{0x0F, 0x12, 0x02, 0x0A},	/*01FF */
	{0x0F, 0x12, 0x00, 0xD4},	/*016E */
	{0x0F, 0x12, 0xFF, 0x55},	/*FF56 */
	{0x0F, 0x12, 0x01, 0xE8},	/*01BD */

	{0x0F, 0x12, 0x02, 0x04},	/*01C5 */
	{0x0F, 0x12, 0xFF, 0xA2},	/*FFB8 */
	{0x0F, 0x12, 0xFF, 0xF3},	/*FFEA */
	{0x0F, 0x12, 0x00, 0xDA},	/*0199 */
	{0x0F, 0x12, 0x01, 0x54},	/*00DE */
	{0x0F, 0x12, 0xFF, 0x6D},	/*FEDE */
	{0x0F, 0x12, 0xFE, 0xF9},	/*FF58 */
	{0x0F, 0x12, 0x02, 0x09},	/*0214 */
	{0x0F, 0x12, 0xFF, 0x3C},	/*FF2C */
	{0x0F, 0x12, 0xFF, 0x2C},	/*FEBC */
	{0x0F, 0x12, 0x00, 0xE7},	/*01CC */
	{0x0F, 0x12, 0x00, 0xA5},	/*00BC */
	{0x0F, 0x12, 0xFF, 0xBE},	/*FFE4 */
	{0x0F, 0x12, 0xFF, 0xDE},	/*0004 */
	{0x0F, 0x12, 0x02, 0x0A},	/*01FF */
	{0x0F, 0x12, 0x00, 0xD4},	/*016E */
	{0x0F, 0x12, 0xFF, 0x55},	/*FF56 */
	{0x0F, 0x12, 0x01, 0xE8},	/*01BD */


	{0x0F, 0x12, 0x02, 0x04},	/*01C5 */
	{0x0F, 0x12, 0xFF, 0xA2},	/*FFB8 */
	{0x0F, 0x12, 0xFF, 0xF3},	/*FFEA */
	{0x0F, 0x12, 0x00, 0xDA},	/*0199 */
	{0x0F, 0x12, 0x01, 0x54},	/*00DE */
	{0x0F, 0x12, 0xFF, 0x6D},	/*FEDE */
	{0x0F, 0x12, 0xFE, 0xF9},	/*FF58 */
	{0x0F, 0x12, 0x02, 0x09},	/*0214 */
	{0x0F, 0x12, 0xFF, 0x3C},	/*FF2C */
	{0x0F, 0x12, 0xFF, 0x2C},	/*FEBC */
	{0x0F, 0x12, 0x00, 0xE7},	/*01CC */
	{0x0F, 0x12, 0x00, 0xA5},	/*00BC */
	{0x0F, 0x12, 0xFF, 0xBE},	/*FFE4 */
	{0x0F, 0x12, 0xFF, 0xDE},	/*0004 */
	{0x0F, 0x12, 0x02, 0x0A},	/*01FF */
	{0x0F, 0x12, 0x00, 0xD4},	/*016E */
	{0x0F, 0x12, 0xFF, 0x55},	/*FF56 */
	{0x0F, 0x12, 0x01, 0xE8},	/*01BD */

	{0x0F, 0x12, 0x02, 0x04},	/*0173 */
	{0x0F, 0x12, 0xFF, 0xA2},	/*FFCE */
	{0x0F, 0x12, 0xFF, 0xF3},	/*FFF8 */
	{0x0F, 0x12, 0x00, 0xDA},	/*00DC */
	{0x0F, 0x12, 0x01, 0x54},	/*00F7 */
	{0x0F, 0x12, 0xFF, 0x6D},	/*FF6F */
	{0x0F, 0x12, 0xFE, 0xF9},	/*FEF0 */
	{0x0F, 0x12, 0x02, 0x09},	/*01CA */
	{0x0F, 0x12, 0xFF, 0x3C},	/*FFA2 */
	{0x0F, 0x12, 0xFF, 0x2C},	/*FEDF */
	{0x0F, 0x12, 0x00, 0xE7},	/*0172 */
	{0x0F, 0x12, 0x00, 0xA5},	/*00C6 */
	{0x0F, 0x12, 0xFF, 0xBE},	/*FFE8 */
	{0x0F, 0x12, 0xFF, 0xDE},	/*FFE8 */
	{0x0F, 0x12, 0x02, 0x0A},	/*01A4 */
	{0x0F, 0x12, 0x00, 0xD4},	/*013E */
	{0x0F, 0x12, 0xFF, 0x55},	/*FF62 */
	{0x0F, 0x12, 0x01, 0xE8},	/*0136 */
	/* param_end	TVAR_wbt_pBaseCcms */

	/* param_start	TVAR_wbt_pOutdoorCcm */
	{0x00, 0x2A, 0x23, 0x80},
	{0x0F, 0x12, 0x01, 0xF0},	/* 019D */
	{0x0F, 0x12, 0xFF, 0xAE},	/* FFC8 */
	{0x0F, 0x12, 0xFF, 0xFA},	/* 000C */
	{0x0F, 0x12, 0x00, 0xF9},	/* 008D */
	{0x0F, 0x12, 0x01, 0x3F},	/* 00CD */
	{0x0F, 0x12, 0xFF, 0x6E},	/* FF3F */
	{0x0F, 0x12, 0xFE, 0xCD},	/* FEDD */
	{0x0F, 0x12, 0x02, 0x30},	/* 01C6 */
	{0x0F, 0x12, 0xFF, 0x7F},	/* FF77 */
	{0x0F, 0x12, 0xFF, 0x37},	/* FEAB */
	{0x0F, 0x12, 0x01, 0xA2},	/* 015D */
	{0x0F, 0x12, 0x01, 0x26},	/* 0082 */
	{0x0F, 0x12, 0xFF, 0xE0},	/* FFCA */
	{0x0F, 0x12, 0xFF, 0xBF},	/* FFA2 */
	{0x0F, 0x12, 0x01, 0xE6},	/* 016F */
	{0x0F, 0x12, 0x01, 0x86},	/* 0171 */
	{0x0F, 0x12, 0xFF, 0x4B},	/* FF35 */
	{0x0F, 0x12, 0x01, 0xB1},	/* 013E */
	/* param_end	TVAR_wbt_pOutdoorCcm */

	{0x00, 0x2A, 0x06, 0xD4},
	{0x0F, 0x12, 0x23, 0x80},
	{0x0F, 0x12, 0x70, 0x00},
	{0x00, 0x2A, 0x06, 0xCC},
	{0x0F, 0x12, 0x23, 0xA4},
	{0x0F, 0x12, 0x70, 0x00},
	{0x00, 0x2A, 0x06, 0xE8},
	{0x0F, 0x12, 0x23, 0xA4},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x23, 0xC8},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x23, 0xEC},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x24, 0x10},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x24, 0x34},
	{0x0F, 0x12, 0x70, 0x00},
	{0x0F, 0x12, 0x24, 0x58},
	{0x0F, 0x12, 0x70, 0x00},

	{0x00, 0x2A, 0x06, 0xDA},
	{0x0F, 0x12, 0x00, 0xBF},	/*SARR_AwbCcmCord[0] 2 700006DA */
	{0x0F, 0x12, 0x00, 0xE6},	/*SARR_AwbCcmCord[1] 2 700006DC */
	{0x0F, 0x12, 0x00, 0xF2},	/*SARR_AwbCcmCord[2] 2 700006DE */
	{0x0F, 0x12, 0x01, 0x43},	/*SARR_AwbCcmCord[3] 2 700006E0 */
	{0x0F, 0x12, 0x01, 0x78},	/*SARR_AwbCcmCord[4] 2 700006E2 */
	{0x0F, 0x12, 0x01, 0xA3},	/*SARR_AwbCcmCord[5] 2 700006E4 */

	/* param_start	SARR_uNormBrInDoor */
	{0x00, 0x2A, 0x07, 0xE8},
	{0x0F, 0x12, 0x00, 0x0A},	/* SARR_uNormBrInDoor[0] */
	{0x0F, 0x12, 0x00, 0x19},	/* SARR_uNormBrInDoor[1] */
	{0x0F, 0x12, 0x00, 0x96},	/* SARR_uNormBrInDoor[2] */
	{0x0F, 0x12, 0x01, 0xF4},	/* SARR_uNormBrInDoor[3] */
	{0x0F, 0x12, 0x07, 0xD0},	/* SARR_uNormBrInDoor[4] */
	/* param_end	SARR_uNormBrInDoor */
	/* param_start	afit_uNoiseIndInDoor */
	{0x00, 0x2A, 0x07, 0xD0},
	{0x0F, 0x12, 0x00, 0x30},	/* afit_uNoiseIndInDoor[0] */
	{0x0F, 0x12, 0x00, 0x46},	/* afit_uNoiseIndInDoor[1] */
	{0x0F, 0x12, 0x00, 0x88},	/* afit_uNoiseIndInDoor[2] */
	{0x0F, 0x12, 0x02, 0x05},	/* afit_uNoiseIndInDoor[3] */
	{0x0F, 0x12, 0x02, 0xBC},	/* afit_uNoiseIndInDoor[4] */
	/* param_end	afit_uNoiseIndInDoor */
	{0x00, 0x2A, 0x07, 0xE6},
	{0x0F, 0x12, 0x00, 0x00},	/*afit_bUseNoiseInd */
	/* param_start	TVAR_afit_pBaseVals */
	{0x00, 0x2A, 0x08, 0x28},
	{0x0F, 0x12, 0x00, 0x10},	/* TVAR_afit_pBaseVals[0] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[1] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[2] */
	{0x0F, 0x12, 0xFF, 0xE2},	/* TVAR_afit_pBaseVals[3] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[4] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[5] */
	{0x0F, 0x12, 0x00, 0xFF},	/* TVAR_afit_pBaseVals[6] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_afit_pBaseVals[7] */
	{0x0F, 0x12, 0x00, 0xFF},	/* TVAR_afit_pBaseVals[8] */
	{0x0F, 0x12, 0x00, 0xFF},	/* TVAR_afit_pBaseVals[9] */
	{0x0F, 0x12, 0x01, 0x29},	/* TVAR_afit_pBaseVals[10] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[11] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_afit_pBaseVals[12] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_afit_pBaseVals[13] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[14] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[15] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[16] */
	{0x0F, 0x12, 0x03, 0x44},	/* TVAR_afit_pBaseVals[17] */
	{0x0F, 0x12, 0x03, 0x3A},	/* TVAR_afit_pBaseVals[18] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[19] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[20] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_afit_pBaseVals[21] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[22] */
	{0x0F, 0x12, 0x00, 0x14},	/* TVAR_afit_pBaseVals[23] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[24] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_afit_pBaseVals[25] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[26] */
	{0x0F, 0x12, 0x00, 0x46},	/* TVAR_afit_pBaseVals[27] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[28] */
	{0x0F, 0x12, 0x01, 0x06},	/* TVAR_afit_pBaseVals[29] */
	{0x0F, 0x12, 0x00, 0x6F},	/* TVAR_afit_pBaseVals[30] */
	{0x0F, 0x12, 0x0C, 0x0F},	/* TVAR_afit_pBaseVals[31] */
	{0x0F, 0x12, 0x0C, 0x0F},	/* TVAR_afit_pBaseVals[32] */
	{0x0F, 0x12, 0x03, 0x03},	/* TVAR_afit_pBaseVals[33] */
	{0x0F, 0x12, 0x03, 0x03},	/* TVAR_afit_pBaseVals[34] */
	{0x0F, 0x12, 0x14, 0x0A},	/* TVAR_afit_pBaseVals[35] */
	{0x0F, 0x12, 0x14, 0x0A},	/* TVAR_afit_pBaseVals[36] */
	{0x0F, 0x12, 0x28, 0x28},	/* TVAR_afit_pBaseVals[37] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[38] */
	{0x0F, 0x12, 0x02, 0x0A},	/* TVAR_afit_pBaseVals[39] */
	{0x0F, 0x12, 0x04, 0x80},	/* TVAR_afit_pBaseVals[40] */
	{0x0F, 0x12, 0x0E, 0x08},	/* TVAR_afit_pBaseVals[41] */
	{0x0F, 0x12, 0x03, 0x0A},	/* TVAR_afit_pBaseVals[42] */
	{0x0F, 0x12, 0x14, 0x03},	/* TVAR_afit_pBaseVals[43] */
	{0x0F, 0x12, 0x0A, 0x11},	/* TVAR_afit_pBaseVals[44] */
	{0x0F, 0x12, 0x0A, 0x0F},	/* TVAR_afit_pBaseVals[45] */
	{0x0F, 0x12, 0x05, 0x0A},	/* TVAR_afit_pBaseVals[46] */
	{0x0F, 0x12, 0x09, 0x14},	/* TVAR_afit_pBaseVals[47] */
	{0x0F, 0x12, 0x00, 0x12},	/* TVAR_afit_pBaseVals[48] */
	{0x0F, 0x12, 0x40, 0x40},	/* TVAR_afit_pBaseVals[49] */
	{0x0F, 0x12, 0x00, 0x05},	/* TVAR_afit_pBaseVals[50] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[51] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[52] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[53] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[54] */
	{0x0F, 0x12, 0x0A, 0x00},	/* TVAR_afit_pBaseVals[55] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[56] */
	{0x0F, 0x12, 0x01, 0x43},	/* TVAR_afit_pBaseVals[57] */
	{0x0F, 0x12, 0x01, 0x4D},	/* TVAR_afit_pBaseVals[58] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_afit_pBaseVals[59] */
	{0x0F, 0x12, 0x80, 0x30},	/* TVAR_afit_pBaseVals[60] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[61] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[62] */
	{0x0F, 0x12, 0x00, 0x10},	/* TVAR_afit_pBaseVals[63] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[64] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[65] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[66] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[67] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[68] */
	{0x0F, 0x12, 0x00, 0x20},	/* TVAR_afit_pBaseVals[69] */
	{0x0F, 0x12, 0x00, 0x0E},	/* TVAR_afit_pBaseVals[70] */
	{0x0F, 0x12, 0x00, 0x80},	/* TVAR_afit_pBaseVals[71] */
	{0x0F, 0x12, 0x00, 0xFF},	/* TVAR_afit_pBaseVals[72] */
	{0x0F, 0x12, 0x01, 0x29},	/* TVAR_afit_pBaseVals[73] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[74] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_afit_pBaseVals[75] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_afit_pBaseVals[76] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[77] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[78] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[79] */
	{0x0F, 0x12, 0x01, 0x14},	/* TVAR_afit_pBaseVals[80] */
	{0x0F, 0x12, 0x02, 0x0A},	/* TVAR_afit_pBaseVals[81] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[82] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[83] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[84] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[85] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[86] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[87] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[88] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[89] */
	{0x0F, 0x12, 0x00, 0xA0},	/* TVAR_afit_pBaseVals[90] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[91] */
	{0x0F, 0x12, 0x01, 0x06},	/* TVAR_afit_pBaseVals[92] */
	{0x0F, 0x12, 0x00, 0x6F},	/* TVAR_afit_pBaseVals[93] */
	{0x0F, 0x12, 0x05, 0x0F},	/* TVAR_afit_pBaseVals[94] */
	{0x0F, 0x12, 0x0A, 0x0F},	/* TVAR_afit_pBaseVals[95] */
	{0x0F, 0x12, 0x02, 0x03},	/* TVAR_afit_pBaseVals[96] */
	{0x0F, 0x12, 0x02, 0x03},	/* TVAR_afit_pBaseVals[97] */
	{0x0F, 0x12, 0x14, 0x0A},	/* TVAR_afit_pBaseVals[98] */
	{0x0F, 0x12, 0x14, 0x0A},	/* TVAR_afit_pBaseVals[99] */
	{0x0F, 0x12, 0x28, 0x28},	/* TVAR_afit_pBaseVals[100] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[101] */
	{0x0F, 0x12, 0x02, 0x0A},	/* TVAR_afit_pBaseVals[102] */
	{0x0F, 0x12, 0x04, 0x80},	/* TVAR_afit_pBaseVals[103] */
	{0x0F, 0x12, 0x0E, 0x08},	/* TVAR_afit_pBaseVals[104] */
	{0x0F, 0x12, 0x03, 0x0A},	/* TVAR_afit_pBaseVals[105] */
	{0x0F, 0x12, 0x0A, 0x03},	/* TVAR_afit_pBaseVals[106] */
	{0x0F, 0x12, 0x0A, 0x11},	/* TVAR_afit_pBaseVals[107] */
	{0x0F, 0x12, 0x32, 0x0F},	/* TVAR_afit_pBaseVals[108] */
	{0x0F, 0x12, 0x05, 0x32},	/* TVAR_afit_pBaseVals[109] */
	{0x0F, 0x12, 0x10, 0x1E},	/* TVAR_afit_pBaseVals[110] */
	{0x0F, 0x12, 0x10, 0x1E},	/* TVAR_afit_pBaseVals[111] */
	{0x0F, 0x12, 0x30, 0x30},	/* TVAR_afit_pBaseVals[112] */
	{0x0F, 0x12, 0x00, 0x05},	/* TVAR_afit_pBaseVals[113] */
	{0x0F, 0x12, 0x04, 0x00},	/* TVAR_afit_pBaseVals[114] */
	{0x0F, 0x12, 0x04, 0x00},	/* TVAR_afit_pBaseVals[115] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[116] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[117] */
	{0x0F, 0x12, 0x0A, 0x00},	/* TVAR_afit_pBaseVals[118] */
	{0x0F, 0x12, 0x10, 0x0A},	/* TVAR_afit_pBaseVals[119] */
	{0x0F, 0x12, 0x01, 0x68},	/* TVAR_afit_pBaseVals[120] */
	{0x0F, 0x12, 0x01, 0x54},	/* TVAR_afit_pBaseVals[121] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_afit_pBaseVals[122] */
	{0x0F, 0x12, 0x80, 0x30},	/* TVAR_afit_pBaseVals[123] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[124] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[125] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[126] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[127] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[128] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[129] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[130] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[131] */
	{0x0F, 0x12, 0x00, 0x0E},	/* TVAR_afit_pBaseVals[132] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_afit_pBaseVals[133] */
	{0x0F, 0x12, 0x00, 0x64},	/* TVAR_afit_pBaseVals[134] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_afit_pBaseVals[135] */
	{0x0F, 0x12, 0x00, 0x02},	/* TVAR_afit_pBaseVals[136] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[137] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[138] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[139] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[140] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[141] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[142] */
	{0x0F, 0x12, 0x00, 0x14},	/* TVAR_afit_pBaseVals[143] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[144] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[145] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[146] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_afit_pBaseVals[147] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[148] */
	{0x0F, 0x12, 0x00, 0x14},	/* TVAR_afit_pBaseVals[149] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[150] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_afit_pBaseVals[151] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[152] */
	{0x0F, 0x12, 0x00, 0x10},	/* TVAR_afit_pBaseVals[153] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[154] */
	{0x0F, 0x12, 0x01, 0x06},	/* TVAR_afit_pBaseVals[155] */
	{0x0F, 0x12, 0x00, 0x6F},	/* TVAR_afit_pBaseVals[156] */
	{0x0F, 0x12, 0x02, 0x02},	/* TVAR_afit_pBaseVals[157] */
	{0x0F, 0x12, 0x05, 0x02},	/* TVAR_afit_pBaseVals[158] */
	{0x0F, 0x12, 0x01, 0x01},	/* TVAR_afit_pBaseVals[159] */
	{0x0F, 0x12, 0x01, 0x02},	/* TVAR_afit_pBaseVals[160] */
	{0x0F, 0x12, 0x14, 0x0A},	/* TVAR_afit_pBaseVals[161] */
	{0x0F, 0x12, 0x14, 0x0A},	/* TVAR_afit_pBaseVals[162] */
	{0x0F, 0x12, 0x28, 0x28},	/* TVAR_afit_pBaseVals[163] */
	{0x0F, 0x12, 0x06, 0x06},	/* TVAR_afit_pBaseVals[164] */
	{0x0F, 0x12, 0x02, 0x05},	/* TVAR_afit_pBaseVals[165] */
	{0x0F, 0x12, 0x04, 0x80},	/* TVAR_afit_pBaseVals[166] */
	{0x0F, 0x12, 0x00, 0x0F},	/* TVAR_afit_pBaseVals[167] */
	{0x0F, 0x12, 0x00, 0x05},	/* TVAR_afit_pBaseVals[168] */
	{0x0F, 0x12, 0x19, 0x03},	/* TVAR_afit_pBaseVals[169] */
	{0x0F, 0x12, 0x19, 0x11},	/* TVAR_afit_pBaseVals[170] */
	{0x0F, 0x12, 0x0A, 0x0F},	/* TVAR_afit_pBaseVals[171] */
	{0x0F, 0x12, 0x05, 0x0A},	/* TVAR_afit_pBaseVals[172] */
	{0x0F, 0x12, 0x20, 0x28},	/* TVAR_afit_pBaseVals[173] */
	{0x0F, 0x12, 0x20, 0x28},	/* TVAR_afit_pBaseVals[174] */
	{0x0F, 0x12, 0x20, 0x00},	/* TVAR_afit_pBaseVals[175] */
	{0x0F, 0x12, 0x00, 0x07},	/* TVAR_afit_pBaseVals[176] */
	{0x0F, 0x12, 0x04, 0x03},	/* TVAR_afit_pBaseVals[177] */
	{0x0F, 0x12, 0x04, 0x02},	/* TVAR_afit_pBaseVals[178] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[179] */
	{0x0F, 0x12, 0x02, 0x03},	/* TVAR_afit_pBaseVals[180] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[181] */
	{0x0F, 0x12, 0x10, 0x06},	/* TVAR_afit_pBaseVals[182] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[183] */
	{0x0F, 0x12, 0x01, 0x73},	/* TVAR_afit_pBaseVals[184] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_afit_pBaseVals[185] */
	{0x0F, 0x12, 0x80, 0x3C},	/* TVAR_afit_pBaseVals[186] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[187] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[188] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[189] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[190] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[191] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[192] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[193] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[194] */
	{0x0F, 0x12, 0x00, 0x08},	/* TVAR_afit_pBaseVals[195] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_afit_pBaseVals[196] */
	{0x0F, 0x12, 0x00, 0x64},	/* TVAR_afit_pBaseVals[197] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_afit_pBaseVals[198] */
	{0x0F, 0x12, 0x00, 0x02},	/* TVAR_afit_pBaseVals[199] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[200] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[201] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[202] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[203] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[204] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[205] */
	{0x0F, 0x12, 0x00, 0x14},	/* TVAR_afit_pBaseVals[206] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[207] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[208] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[209] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_afit_pBaseVals[210] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[211] */
	{0x0F, 0x12, 0x00, 0x14},	/* TVAR_afit_pBaseVals[212] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[213] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_afit_pBaseVals[214] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[215] */
	{0x0F, 0x12, 0x00, 0x10},	/* TVAR_afit_pBaseVals[216] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[217] */
	{0x0F, 0x12, 0x01, 0x06},	/* TVAR_afit_pBaseVals[218] */
	{0x0F, 0x12, 0x00, 0x6F},	/* TVAR_afit_pBaseVals[219] */
	{0x0F, 0x12, 0x02, 0x02},	/* TVAR_afit_pBaseVals[220] */
	{0x0F, 0x12, 0x05, 0x02},	/* TVAR_afit_pBaseVals[221] */
	{0x0F, 0x12, 0x01, 0x01},	/* TVAR_afit_pBaseVals[222] */
	{0x0F, 0x12, 0x01, 0x02},	/* TVAR_afit_pBaseVals[223] */
	{0x0F, 0x12, 0x14, 0x0A},	/* TVAR_afit_pBaseVals[224] */
	{0x0F, 0x12, 0x14, 0x0A},	/* TVAR_afit_pBaseVals[225] */
	{0x0F, 0x12, 0x28, 0x28},	/* TVAR_afit_pBaseVals[226] */
	{0x0F, 0x12, 0x06, 0x06},	/* TVAR_afit_pBaseVals[227] */
	{0x0F, 0x12, 0x02, 0x05},	/* TVAR_afit_pBaseVals[228] */
	{0x0F, 0x12, 0x04, 0x80},	/* TVAR_afit_pBaseVals[229] */
	{0x0F, 0x12, 0x00, 0x0F},	/* TVAR_afit_pBaseVals[230] */
	{0x0F, 0x12, 0x00, 0x05},	/* TVAR_afit_pBaseVals[231] */
	{0x0F, 0x12, 0x28, 0x03},	/* TVAR_afit_pBaseVals[232] */
	{0x0F, 0x12, 0x28, 0x11},	/* TVAR_afit_pBaseVals[233] */
	{0x0F, 0x12, 0x0A, 0x0F},	/* TVAR_afit_pBaseVals[234] */
	{0x0F, 0x12, 0x05, 0x0A},	/* TVAR_afit_pBaseVals[235] */
	{0x0F, 0x12, 0x20, 0x28},	/* TVAR_afit_pBaseVals[236] */
	{0x0F, 0x12, 0x20, 0x28},	/* TVAR_afit_pBaseVals[237] */
	{0x0F, 0x12, 0x20, 0x00},	/* TVAR_afit_pBaseVals[238] */
	{0x0F, 0x12, 0x00, 0x07},	/* TVAR_afit_pBaseVals[239] */
	{0x0F, 0x12, 0x04, 0x03},	/* TVAR_afit_pBaseVals[240] */
	{0x0F, 0x12, 0x04, 0x02},	/* TVAR_afit_pBaseVals[241] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[242] */
	{0x0F, 0x12, 0x02, 0x03},	/* TVAR_afit_pBaseVals[243] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[244] */
	{0x0F, 0x12, 0x10, 0x06},	/* TVAR_afit_pBaseVals[245] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[246] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[247] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_afit_pBaseVals[248] */
	{0x0F, 0x12, 0x80, 0x3C},	/* TVAR_afit_pBaseVals[249] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[250] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[251] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[252] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[253] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[254] */
	{0x0F, 0x12, 0x00, 0x14},	/* TVAR_afit_pBaseVals[255] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[256] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[257] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_afit_pBaseVals[258] */
	{0x0F, 0x12, 0x00, 0x06},	/* TVAR_afit_pBaseVals[259] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_afit_pBaseVals[260] */
	{0x0F, 0x12, 0x00, 0x50},	/* TVAR_afit_pBaseVals[261] */
	{0x0F, 0x12, 0x00, 0x02},	/* TVAR_afit_pBaseVals[262] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[263] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[264] */
	{0x0F, 0x12, 0x00, 0x0A},	/* TVAR_afit_pBaseVals[265] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[266] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[267] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[268] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[269] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[270] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[271] */
	{0x0F, 0x12, 0x03, 0xFF},	/* TVAR_afit_pBaseVals[272] */
	{0x0F, 0x12, 0x00, 0x28},	/* TVAR_afit_pBaseVals[273] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[274] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[275] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[276] */
	{0x0F, 0x12, 0x00, 0x30},	/* TVAR_afit_pBaseVals[277] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[278] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[279] */
	{0x0F, 0x12, 0x00, 0x32},	/* TVAR_afit_pBaseVals[280] */
	{0x0F, 0x12, 0x01, 0x06},	/* TVAR_afit_pBaseVals[281] */
	{0x0F, 0x12, 0x00, 0x6F},	/* TVAR_afit_pBaseVals[282] */
	{0x0F, 0x12, 0x02, 0x02},	/* TVAR_afit_pBaseVals[283] */
	{0x0F, 0x12, 0x05, 0x02},	/* TVAR_afit_pBaseVals[284] */
	{0x0F, 0x12, 0x01, 0x01},	/* TVAR_afit_pBaseVals[285] */
	{0x0F, 0x12, 0x01, 0x01},	/* TVAR_afit_pBaseVals[286] */
	{0x0F, 0x12, 0xFF, 0xFF},	/* TVAR_afit_pBaseVals[287] */
	{0x0F, 0x12, 0x14, 0x0A},	/* TVAR_afit_pBaseVals[288] */
	{0x0F, 0x12, 0x28, 0x28},	/* TVAR_afit_pBaseVals[289] */
	{0x0F, 0x12, 0x06, 0x06},	/* TVAR_afit_pBaseVals[290] */
	{0x0F, 0x12, 0x02, 0x05},	/* TVAR_afit_pBaseVals[291] */
	{0x0F, 0x12, 0x08, 0x80},	/* TVAR_afit_pBaseVals[292] */
	{0x0F, 0x12, 0x00, 0x0F},	/* TVAR_afit_pBaseVals[293] */
	{0x0F, 0x12, 0x00, 0x05},	/* TVAR_afit_pBaseVals[294] */
	{0x0F, 0x12, 0x28, 0x03},	/* TVAR_afit_pBaseVals[295] */
	{0x0F, 0x12, 0x28, 0x11},	/* TVAR_afit_pBaseVals[296] */
	{0x0F, 0x12, 0x0A, 0x0F},	/* TVAR_afit_pBaseVals[297] */
	{0x0F, 0x12, 0x05, 0x0A},	/* TVAR_afit_pBaseVals[298] */
	{0x0F, 0x12, 0x20, 0x20},	/* TVAR_afit_pBaseVals[299] */
	{0x0F, 0x12, 0x20, 0x20},	/* TVAR_afit_pBaseVals[300] */
	{0x0F, 0x12, 0x64, 0x00},	/* TVAR_afit_pBaseVals[301] */
	{0x0F, 0x12, 0x00, 0x07},	/* TVAR_afit_pBaseVals[302] */
	{0x0F, 0x12, 0x04, 0x08},	/* TVAR_afit_pBaseVals[303] */
	{0x0F, 0x12, 0x04, 0x06},	/* TVAR_afit_pBaseVals[304] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[305] */
	{0x0F, 0x12, 0x06, 0x08},	/* TVAR_afit_pBaseVals[306] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[307] */
	{0x0F, 0x12, 0x10, 0x06},	/* TVAR_afit_pBaseVals[308] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[309] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[310] */
	{0x0F, 0x12, 0x01, 0x00},	/* TVAR_afit_pBaseVals[311] */
	{0x0F, 0x12, 0x80, 0x50},	/* TVAR_afit_pBaseVals[312] */
	{0x0F, 0x12, 0x01, 0x80},	/* TVAR_afit_pBaseVals[313] */
	{0x0F, 0x12, 0x00, 0x00},	/* TVAR_afit_pBaseVals[314] */
	/* param_end	TVAR_afit_pBaseVals */
	/* param_start	afit_pConstBaseVals */
	{0x0F, 0x12, 0x00, 0xFF},	/*afit_pConstBaseVals[0] */
	{0x0F, 0x12, 0x00, 0xFF},	/*afit_pConstBaseVals[1] */
	{0x0F, 0x12, 0x08, 0x00},	/*afit_pConstBaseVals[2] */
	{0x0F, 0x12, 0x06, 0x00},	/*afit_pConstBaseVals[3] */
	{0x0F, 0x12, 0x00, 0x00},	/*afit_pConstBaseVals[4] */
	{0x0F, 0x12, 0x00, 0x00},	/*afit_pConstBaseVals[5] */
	{0x0F, 0x12, 0x00, 0x00},	/*afit_pConstBaseVals[6] */
	{0x0F, 0x12, 0x03, 0x00},	/*afit_pConstBaseVals[7] */
	{0x0F, 0x12, 0x00, 0x02},	/*afit_pConstBaseVals[8] */
	{0x0F, 0x12, 0x04, 0x00},	/*afit_pConstBaseVals[9] */
	{0x0F, 0x12, 0x01, 0x06},	/*afit_pConstBaseVals[10] */
	{0x0F, 0x12, 0x00, 0x05},	/*afit_pConstBaseVals[11] */
	{0x0F, 0x12, 0x00, 0x00},	/*afit_pConstBaseVals[12] */
	{0x0F, 0x12, 0x00, 0x03},	/*afit_pConstBaseVals[13] */
	{0x0F, 0x12, 0x00, 0x00},	/*afit_pConstBaseVals[14] */
	{0x0F, 0x12, 0xFF, 0xD6},	/*afit_pConstBaseVals[15] */
	{0x0F, 0x12, 0x53, 0xC1},	/*afit_pConstBaseVals[16] */
	{0x0F, 0x12, 0xE1, 0xFE},	/*afit_pConstBaseVals[17] */
	{0x0F, 0x12, 0x00, 0x01},	/*afit_pConstBaseVals[18] */
	/* param_end	afit_pConstBaseVals */

	{0x00, 0x2A, 0x04, 0x88},
	{0x0F, 0x12, 0x41, 0x6E},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0xA3, 0x16},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x20, 0x5C},
	{0x0F, 0x12, 0xF4, 0x24},
	{0x0F, 0x12, 0x00, 0x00},

	{0x00, 0x2A, 0x04, 0x90},
	{0x0F, 0x12, 0x41, 0x6E},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0xA3, 0x16},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x20, 0x60},
	{0x0F, 0x12, 0xF4, 0x24},
	{0x0F, 0x12, 0x00, 0x00},

	{0x00, 0x2A, 0x04, 0x98},
	{0x0F, 0x12, 0x01, 0x80},
	{0x0F, 0x12, 0x02, 0x58},
	{0x00, 0x2A, 0x20, 0x64},
	{0x0F, 0x12, 0x05, 0x00},
	{0x00, 0x2A, 0x04, 0x9C},
	{0x0F, 0x12, 0x01, 0x60},


	/* WRITE #AWBBTune_EVT4_uMaxExp3     0000 */
	/* WRITE #AWBBTune_EVT4_uCapMaxExp3  0000 */
	/* WRITE #AWBBTune_EVT4_uMaxAnGain3  0000 */

	{0x00, 0x2A, 0x20, 0x58},
	{0x0F, 0x12, 0x00, 0x01},
	{0x0F, 0x12, 0x00, 0x90},


	{0x00, 0x2A, 0x10, 0x0E},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x02, 0x01},
	{0x0F, 0x12, 0x02, 0x02},
	{0x0F, 0x12, 0x02, 0x02},
	{0x0F, 0x12, 0x01, 0x02},
	{0x0F, 0x12, 0x02, 0x01},
	{0x0F, 0x12, 0x03, 0x02},
	{0x0F, 0x12, 0x02, 0x03},
	{0x0F, 0x12, 0x01, 0x02},
	{0x0F, 0x12, 0x02, 0x01},
	{0x0F, 0x12, 0x03, 0x02},
	{0x0F, 0x12, 0x02, 0x03},
	{0x0F, 0x12, 0x01, 0x02},
	{0x0F, 0x12, 0x02, 0x01},
	{0x0F, 0x12, 0x03, 0x02},
	{0x0F, 0x12, 0x02, 0x03},
	{0x0F, 0x12, 0x01, 0x02},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x02, 0x02},
	{0x0F, 0x12, 0x02, 0x02},
	{0x0F, 0x12, 0x00, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},
	{0x0F, 0x12, 0x01, 0x01},

	/* End tuning part */
	/* Set host interrupt so main start run */
	{0x10, 0x00, 0x00, 0x01},

	/********************/
	/* Delay{100}, p10 */
	/* Wait10mSec */
	/* Start user init script */

	/* End user init script */

	{REG_DELAY, 10, 0, 0},

	{0x00, 0x2A, 0x04, 0x00},
	{0x0F, 0x12, 0x00, 0x5F},
	{0x00, 0x2A, 0x03, 0xDC},
	{0x0F, 0x12, 0x00, 0x02},
	{0x0F, 0x12, 0x00, 0x01},
	/********************/
	/* clk Settings */

	{0x00, 0x2A, 0x01, 0xB8},
	{0x0F, 0x12, 0x5D, 0xC0},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x01, 0xC6},
	{0x0F, 0x12, 0x00, 0x01},
	{0x0F, 0x12, 0x00, 0x01},
	{0x00, 0x2A, 0x01, 0xCC},
	{0x0F, 0x12, 0x1B, 0x58},
	{0x0F, 0x12, 0x36, 0xB0},
	{0x0F, 0x12, 0x36, 0xB0},
	{0x0F, 0x12, 0x1B, 0x58},
	{0x0F, 0x12, 0x36, 0xB0},
	{0x0F, 0x12, 0x36, 0xB0},
	{0x00, 0x2A, 0x01, 0xE0},
	{0x0F, 0x12, 0x00, 0x01},

	/********************/
	/* Delay{1000},p100 */
	{REG_DELAY, 100, 0, 0},

	/*
	 * PREVIEW CONFIGURATION 0
	 * {SXGA, YUV, 15fps}
	 */
	{0x00, 0x2A, 0x02, 0x42},
	{0x0F, 0x12, 0x05, 0x00},	/* 1280 */
	{0x0F, 0x12, 0x04, 0x00},	/* 1024 */
	{0x0F, 0x12, 0x00, 0x05},	/* YUV */
	{0x00, 0x2A, 0x02, 0x4E},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x02, 0x48},
	{0x0F, 0x12, 0x36, 0xB0},
	{0x0F, 0x12, 0x36, 0xB0},
	{0x0F, 0x12, 0x00, 0x42},
	{0x00, 0x2A, 0x02, 0x52},
	{0x0F, 0x12, 0x00, 0x02},
	{0x00, 0x2A, 0x02, 0x50},
	{0x0F, 0x12, 0x00, 0x02},
	{0x00, 0x2A, 0x02, 0x54},
	{0x0F, 0x12, 0x02, 0x9A},
	{0x0F, 0x12, 0x00, 0x00},

	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x00},

	/*
	 * PREVIEW CONFIGURATION 1
	 * {VGA, YUV, 30fps}
	 */

	{0x0F, 0x12, 0x02, 0x80},	/* 640 */
	{0x0F, 0x12, 0x01, 0xE0},	/* 480 */
	{0x0F, 0x12, 0x00, 0x05},	/* YUV */
	{0x00, 0x2A, 0x02, 0x74},
	{0x0F, 0x12, 0x00, 0x01},
	{0x00, 0x2A, 0x02, 0x6E},
	{0x0F, 0x12, 0x36, 0xB0},
	{0x0F, 0x12, 0x00, 0x00},
	{0x0F, 0x12, 0x00, 0x52},
	{0x00, 0x2A, 0x02, 0x78},
	{0x0F, 0x12, 0x00, 0x01},
	{0x00, 0x2A, 0x02, 0x76},
	{0x0F, 0x12, 0x00, 0x02},
	{0x00, 0x2A, 0x02, 0x7A},
	{0x0F, 0x12, 0x01, 0x4D},
	{0x0F, 0x12, 0x00, 0x00},

	/********************/
	/* PREVIEW */

	{0x00, 0x2A, 0x03, 0xB6},
	{0x0F, 0x12, 0x00, 0x00},
	{0x00, 0x2A, 0x03, 0xFA},
	{0x0F, 0x12, 0x00, 0x01},
	{0x0F, 0x12, 0x00, 0xC3},
	{0x0F, 0x12, 0x00, 0x01},


	{0x00, 0x2A, 0x02, 0x1C},
	{0x0F, 0x12, 0x00, 0x01},
	{0x00, 0x2A, 0x02, 0x20},
	{0x0F, 0x12, 0x00, 0x01},
	{0x00, 0x2A, 0x01, 0xF8},
	{0x0F, 0x12, 0x00, 0x01},
	{0x00, 0x2A, 0x02, 0x1E},
	{0x0F, 0x12, 0x00, 0x01},
	{0x00, 0x2A, 0x01, 0xF0},
	{0x0F, 0x12, 0x00, 0x01},
	{0x0F, 0x12, 0x00, 0x01},
	/********************/

	/* Delay{2000} */
	{REG_DELAY, 200, 0, 0},
};

#define S5K6AA_INIT_REGS	\
	(sizeof(s5k6aa_init_reg) / sizeof(s5k6aa_init_reg[0]))

unsigned short s5k6aa_sleep_reg[][2] = {
	{0x0028, 0x7000},
	{0x002A, setot_usForceClocksSettings},
	{0x0F12, 0x001E},
	{0x002A, setot_usConfigClocksSettings},
	{0x0F12, 0x001E},

	{0x002A, REG_TC_GP_EnablePreview},
	{0x0F12, 0x0000},
	{0x002A, REG_TC_GP_EnablePreviewChanged},
	{0x0F12, 0x0001},

	{REG_DELAY, 100},

	{0x0028, 0xD000},
	{0x002A, 0xB0B0},
	{0x0F12, 0x0001},
	{REG_DELAY, 100},

	{0x002A, 0xB0B4},
	{0x0F12, 0x0000},

	{0x002A, 0xB0A0},
	{0x0F12, 0x0031},

	{0x002A, 0xB0A0},
	{0x0F12, 0x0010},

	{0x002A, 0xB0A0},
	{0x0F12, 0x0000},

	{0x0028, 0x7000},
	{0x002A, REG_TC_GP_SleepMode},
	{0x0F12, 0x0001},
	{0x002A, REG_TC_GP_SleepModeChanged},
	{0x0F12, 0x0001},
};

#define S5K6AA_SLEEP_REGS	\
	(sizeof(s5k6aa_sleep_reg) / sizeof(s5k6aa_sleep_reg[0]))

unsigned short s5k6aa_wakeup_reg[][2] = {
	{0x0028, 0x7000},
	{0x002A, REG_TC_GP_SleepMode},
	{0x0F12, 0x0000},
	{0x002A, REG_TC_GP_SleepModeChanged},
	{0x0F12, 0x0001},

	{0x0028, 0xD000},
	{0x002A, 0x1000},
	{0x0F12, 0x0001},

	{0x002A, 0x003A},
	{0x0F12, 0x0000},

	{REG_DELAY, 200},

	{0x002A, 0xB0B0},
	{0x0F12, 0x0000},

	{0x002A, 0xB0A0},
	{0x0F12, 0x003B},

	{0x002A, 0xB0B4},
	{0x0F12, 0x0800},

	{REG_DELAY, 10},

	{0x0028, 0x7000},
	{0x002A, REG_TC_GP_EnablePreview},
	{0x0F12, 0x0001},
	{0x002A, REG_TC_GP_EnablePreviewChanged},
	{0x0F12, 0x0001},
};

#define S5K6AA_WAKEUP_REGS	\
	(sizeof(s5k6aa_wakeup_reg) / sizeof(s5k6aa_wakeup_reg[0]))

/* Clock configuration: should be expanded to various clock input */
static const struct s5k6aa_reg s5k6aa_mclk_24mhz[] = {
	{0x002A, REG_TC_IPRM_InClockLSBs},
	{0x0F12, 0x5DC0},	/* input=24MHz */
	{0x002A, REG_TC_IPRM_InClockMSBs},
	{0x0F12, 0x0000},
	{0x002A, REG_TC_IPRM_UseNPviClocks},
	{0x0F12, 0x0001},	/* 1 PLL configurations */
	{0x002A, REG_TC_IPRM_OpClk4KHz_0},
	{0x0F12, 0x1770},	/* 1st system CLK 24MHz */
	{0x002A, REG_TC_IPRM_MinOutRate4KHz_0},
	{0x0F12, 0x2EE0},
	{0x002A, REG_TC_IPRM_MaxOutRate4KHz_0},
	{0x0F12, 0x2EE0},
	{0x002A, REG_TC_IPRM_InitParamsUpdated},
	{0x0F12, 0x0001},	/* FIXME:Preset number ??? */
	{REG_DELAY, 100},	/* 100ms delay */
};

/*
 * Image resolution configuration
 * This SoC supports 5 presets for each preview and capture resolutions
 */
/* Preview configuration preset #0 */
static const struct s5k6aa_reg s5k6aa_preview_vga[] = {
	/* Resolution, Pixel format */
	{0x002A, REG_0TC_PCFG_usWidth},
	{0x0F12, 0x0280},	/* 640 */
	{0x002A, REG_0TC_PCFG_usHeight},
	{0x0F12, 0x01E0},	/* 480 */
	{0x002A, REG_0TC_PCFG_Format},
	{0x0F12, 0x0005},	/* YUV */

	/* PLL config */
	{0x002A, REG_0TC_PCFG_uClockInd},
	{0x0F12, 0x0000},

	/* PCLK max */
	{0x002A, REG_0TC_PCFG_usMaxOut4KHzRate},
	{0x0F12, 0x2EE0},
	/* PCLK min */
	{0x002A, REG_0TC_PCFG_usMinOut4KHzRate},
	{0x0F12, 0x2EE0},
	{0x002A, REG_0TC_PCFG_PVIMask},
	{0x0F12, 0x0042},

	/* Frame rate */
	{0x002A, REG_0TC_PCFG_FrRateQualityType},
	{0x0F12, TC_FRVSQ_BEST_QUALITY},	/* binning disabled */
	{0x002A, REG_0TC_PCFG_usFrTimeType},
	{0x0F12, TC_FR_TIME_FIXED_ACCURATE},	/* Fixed */
	{0x002A, REG_0TC_PCFG_uPrevMirror},
	{0x0F12, 0x0000},	/* no mirror */
	/* max frame time : 15fps 029a */
	{0x002A, REG_0TC_PCFG_usMaxFrTimeMsecMult10},
	{0x0F12, 0x0535},
	{0x002A, REG_0TC_PCFG_usMinFrTimeMsecMult10},
	{0x0F12, 0x0000},
};
/* Preview configuration preset #1 */
/* Preview configuration preset #2 */
/* Preview configuration preset #3 */
/* Preview configuration preset #4 */

/* Capture configuration preset #0 */
/* Capture configuration preset #1 */
/* Capture configuration preset #2 */
/* Capture configuration preset #3 */
/* Capture configuration preset #4 */

/*
 * EV bias
 */

static const struct s5k6aa_reg s5k6aa_ev_m6[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_m5[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_m4[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_m3[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_m2[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_m1[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_default[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_p1[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_p2[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_p3[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_p4[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_p5[] = {
};

static const struct s5k6aa_reg s5k6aa_ev_p6[] = {
};

#ifdef S5K6AA_COMPLETE
/* Order of this array should be following the querymenu data */
static const unsigned char *s5k6aa_regs_ev_bias[] = {
	(unsigned char *)s5k6aa_ev_m6, (unsigned char *)s5k6aa_ev_m5,
	(unsigned char *)s5k6aa_ev_m4, (unsigned char *)s5k6aa_ev_m3,
	(unsigned char *)s5k6aa_ev_m2, (unsigned char *)s5k6aa_ev_m1,
	(unsigned char *)s5k6aa_ev_default, (unsigned char *)s5k6aa_ev_p1,
	(unsigned char *)s5k6aa_ev_p2, (unsigned char *)s5k6aa_ev_p3,
	(unsigned char *)s5k6aa_ev_p4, (unsigned char *)s5k6aa_ev_p5,
	(unsigned char *)s5k6aa_ev_p6,
};

/*
 * Auto White Balance configure
 */
static const struct s5k6aa_reg s5k6aa_awb_off[] = {
};

static const struct s5k6aa_reg s5k6aa_awb_on[] = {
};

static const unsigned char *s5k6aa_regs_awb_enable[] = {
	(unsigned char *)s5k6aa_awb_off,
	(unsigned char *)s5k6aa_awb_on,
};

/*
 * Manual White Balance (presets)
 */
static const struct s5k6aa_reg s5k6aa_wb_tungsten[] = {

};

static const struct s5k6aa_reg s5k6aa_wb_fluorescent[] = {

};

static const struct s5k6aa_reg s5k6aa_wb_sunny[] = {

};

static const struct s5k6aa_reg s5k6aa_wb_cloudy[] = {

};

/* Order of this array should be following the querymenu data */
static const unsigned char *s5k6aa_regs_wb_preset[] = {
	(unsigned char *)s5k6aa_wb_tungsten,
	(unsigned char *)s5k6aa_wb_fluorescent,
	(unsigned char *)s5k6aa_wb_sunny,
	(unsigned char *)s5k6aa_wb_cloudy,
};

/*
 * Color Effect (COLORFX)
 */
static const struct s5k6aa_reg s5k6aa_color_sepia[] = {
};

static const struct s5k6aa_reg s5k6aa_color_aqua[] = {
};

static const struct s5k6aa_reg s5k6aa_color_monochrome[] = {
};

static const struct s5k6aa_reg s5k6aa_color_negative[] = {
};

static const struct s5k6aa_reg s5k6aa_color_sketch[] = {
};

/* Order of this array should be following the querymenu data */
static const unsigned char *s5k6aa_regs_color_effect[] = {
	(unsigned char *)s5k6aa_color_sepia,
	(unsigned char *)s5k6aa_color_aqua,
	(unsigned char *)s5k6aa_color_monochrome,
	(unsigned char *)s5k6aa_color_negative,
	(unsigned char *)s5k6aa_color_sketch,
};

/*
 * Contrast bias
 */
static const struct s5k6aa_reg s5k6aa_contrast_m2[] = {
};

static const struct s5k6aa_reg s5k6aa_contrast_m1[] = {
};

static const struct s5k6aa_reg s5k6aa_contrast_default[] = {
};

static const struct s5k6aa_reg s5k6aa_contrast_p1[] = {
};

static const struct s5k6aa_reg s5k6aa_contrast_p2[] = {
};

static const unsigned char *s5k6aa_regs_contrast_bias[] = {
	(unsigned char *)s5k6aa_contrast_m2,
	(unsigned char *)s5k6aa_contrast_m1,
	(unsigned char *)s5k6aa_contrast_default,
	(unsigned char *)s5k6aa_contrast_p1,
	(unsigned char *)s5k6aa_contrast_p2,
};

/*
 * Saturation bias
 */
static const struct s5k6aa_reg s5k6aa_saturation_m2[] = {
};

static const struct s5k6aa_reg s5k6aa_saturation_m1[] = {
};

static const struct s5k6aa_reg s5k6aa_saturation_default[] = {
};

static const struct s5k6aa_reg s5k6aa_saturation_p1[] = {
};

static const struct s5k6aa_reg s5k6aa_saturation_p2[] = {
};

static const unsigned char *s5k6aa_regs_saturation_bias[] = {
	(unsigned char *)s5k6aa_saturation_m2,
	(unsigned char *)s5k6aa_saturation_m1,
	(unsigned char *)s5k6aa_saturation_default,
	(unsigned char *)s5k6aa_saturation_p1,
	(unsigned char *)s5k6aa_saturation_p2,
};

/*
 * Sharpness bias
 */
static const struct s5k6aa_reg s5k6aa_sharpness_m2[] = {
};

static const struct s5k6aa_reg s5k6aa_sharpness_m1[] = {
};

static const struct s5k6aa_reg s5k6aa_sharpness_default[] = {
};

static const struct s5k6aa_reg s5k6aa_sharpness_p1[] = {
};

static const struct s5k6aa_reg s5k6aa_sharpness_p2[] = {
};

static const unsigned char *s5k6aa_regs_sharpness_bias[] = {
	(unsigned char *)s5k6aa_sharpness_m2,
	(unsigned char *)s5k6aa_sharpness_m1,
	(unsigned char *)s5k6aa_sharpness_default,
	(unsigned char *)s5k6aa_sharpness_p1,
	(unsigned char *)s5k6aa_sharpness_p2,
};
#endif /* S5K6AA_COMPLETE */

#endif
